<?php
defined('BASEPATH') or exit('No direct script access allowed');
/*
  Widget Name: Staff Breaks with Details
  Description: Display staff breaks with their full names
*/
?>

<?php
$fn_get_data = function () {
    $widget_req_from = $this->input->get('period_from');
    if (!empty($widget_req_from)) {
        $widget_req_from = DateTime::createFromFormat('Y-m-d', $widget_req_from);
        if ($widget_req_from !== false) {
            $widget_req_from = $widget_req_from->format('Y-m-d');
        } else {
            $widget_req_from = null;
        }
    } else {
        $widget_req_from = null;
    }

    $widget_req_to = $this->input->get('period_to');
    if (!empty($widget_req_to)) {
        $widget_req_to = DateTime::createFromFormat('Y-m-d', $widget_req_to);
        if ($widget_req_to !== false) {
            $widget_req_to = $widget_req_to->format('Y-m-d');
        } else {
            $widget_req_to = null;
        }
    } else {
        $widget_req_to = null;
    }

    // SQL query to get all staff who are currently on break
    $sql = "
        SELECT 
            Breaks.id,
            Breaks.staff_id,
            CONCAT(staff.firstname, ' ', staff.lastname) AS staff_fullname,
            Breaks.break_type,
            Breaks.break_minutes,
            Breaks.start_time,
            Breaks.end_time,
            Breaks.on_break,
            Breaks.mins_late,
            Breaks.created_at
        FROM 
            " . db_prefix() . "breaks Breaks
        JOIN 
            " . db_prefix() . "staff staff ON Breaks.staff_id = staff.staffid
        WHERE 
            Breaks.on_break = 1
        ORDER BY Breaks.created_at DESC
    ";

    return $this->db->query($sql)->result_array();
};

// Get all staff who are on break
$widget_data = $fn_get_data();

?>


<style>
/* Blinking red effect */
.blink-row {
    animation: blink-animation 1s infinite;
}

@keyframes blink-animation {
    0% { background-color: red; }
    50% { background-color: transparent; }
    100% { background-color: red; }
}


</style>


<div class="widget widget-staff-breaks widget-<?= $widget['id'] ?>" data-widget-id="<?= $widget['id'] ?>">
    <div class="">
        <div class="panel_s">
            <div class="panel-body">
                <div class="widget-dragger"></div>

                <h4 class="pull-left mtop5"><?php echo _l('staff_breaks_with_details'); ?></h4>
                <div class="clearfix"></div>
                <div class="row mtop5">
                    <hr class="hr-panel-heading-dashboard">
                </div>

                <table class="table dataTable no-footer dtr-inline" id="staff-breaks-table">
                    <thead>
                        <th><?= _l('staff') ?></th>
                        <th><?= _l('break_type') ?></th>
                        <th><?= _l('break_minutes') ?></th>
                        <th><?= _l('start_time') ?></th>
                        <th><?= _l('end_time') ?></th>
                        <th><?= _l('on_break') ?></th>
                        <th><?= _l('mins_late') ?></th>
                        <th><?= _l('created_at') ?></th>
                    </thead>
                    <tbody>
                        <?php if (count($widget_data) > 0) { ?>
                            <?php foreach ($widget_data as $widget_row) { ?>
                                <tr data-staff-id="<?= $widget_row['staff_id'] ?>" data-end-time="<?= $widget_row['end_time'] ?>">
                                    <td><a href="<?= admin_url('staff/member/' . $widget_row['staff_id']) ?>"><?= $widget_row['staff_fullname'] ?></a></td>
                                    <td><?= $widget_row['break_type'] ?></td>
                                    <td><?= $widget_row['break_minutes'] ?></td>
                                    <td><?= $widget_row['start_time'] ?></td>
                                    <td><?= $widget_row['end_time'] ?></td>
                                    <td><?= $widget_row['on_break'] ? _l('yes') : _l('no') ?></td>
                                    <td><?= $widget_row['mins_late'] ?></td>
                                    <td><?= _dt($widget_row['created_at']) ?></td>
                                </tr>
                            <?php } ?>
                        <?php } else { ?>
                            <tr>
                                <td colspan="8"><?= _l('not_found') ?></td>
                            </tr>
                        <?php } ?>
                    </tbody>
                </table>
            </div>
        </div>
    </div>
</div>


<script>
    // Poll every 5 seconds to check for exceeded breaks
    setInterval(function() {
        const now = new Date();

        // Iterate over each row in the table
        $('#staff-breaks-table tbody tr').each(function() {
            var row = $(this);
            var endTime = row.data('end-time'); // Get the end time for each staff
            
            // Convert endTime to a Date object
            var endTimeDate = new Date(endTime);

            // Check if the current time exceeds the end time
            if (now > endTimeDate) {
                row.addClass('blink-row'); // Add blinking class if break is exceeded
            } else {
                row.removeClass('blink-row'); // Remove blinking if within time
            }
        });
    }, 5000); // Poll every 5 seconds
</script>


