<?php

defined('BASEPATH') or exit('No direct script access allowed');
/*
  Widget Name: Staff with roles
  Description: Staff with roles
*/
?>

<?php

$fn_get_data = function () {
    // Validate 'period_from' input
    $widget_req_from = $this->input->get('period_from');
    if (!empty($widget_req_from)) {
        $widget_req_from = DateTime::createFromFormat('Y-m-d', $widget_req_from);
        if ($widget_req_from !== false) {
            $widget_req_from = $widget_req_from->format('Y-m-d');
        } else {
            $widget_req_from = null;
        }
    } else {
        $widget_req_from = null;
    }

    // Validate 'period_to' input
    $widget_req_to = $this->input->get('period_to');
    if (!empty($widget_req_to)) {
        $widget_req_to = DateTime::createFromFormat('Y-m-d', $widget_req_to);
        if ($widget_req_to !== false) {
            $widget_req_to = $widget_req_to->format('Y-m-d');
        } else {
            $widget_req_to = null;
        }
    } else {
        $widget_req_to = null;
    }

    // SQL query to get staff with roles
    $sql = "
        SELECT 
            TBLRoles.roleid AS role_id,
            TBLRoles.name AS role_name,
            COUNT(TBLStaff.staffid) AS total_staff
        FROM
            " . db_prefix() . "roles TBLRoles
            LEFT JOIN " . db_prefix() . "staff TBLStaff ON TBLRoles.roleid = TBLStaff.role
    ";

    // Apply date range filters if both dates are provided
    if (isset($widget_req_from) && isset($widget_req_to)) {
        $sql .= "
            WHERE TBLStaff.datecreated >= '" . $widget_req_from . " 00:00:00' 
            AND TBLStaff.datecreated <= '" . $widget_req_to . " 23:59:59'
        ";
    }

    $sql .= "
        GROUP BY role_id, role_name
        ORDER BY total_staff DESC
        LIMIT 0, 10
    ";

    $resultArray = $this->db->query($sql)->result_array();
    $colors = get_system_favourite_colors();

    // Prepare the chart data
    $chart = [
        'labels' => [],
        'datasets' => [
            [
                'data' => [],
                'backgroundColor' => [],
                'hoverBackgroundColor' => [],
            ],
        ],
    ];

    foreach ($resultArray as $result_num => $result_row) {
        if ($result_row['total_staff'] > 0) {
            array_push($chart['labels'], $result_row['role_name']);
            array_push($chart['datasets'][0]['data'], $result_row['total_staff']);
            array_push($chart['datasets'][0]['backgroundColor'], $colors[$result_num % count($colors)]);
            array_push($chart['datasets'][0]['hoverBackgroundColor'], adjust_color_brightness($colors[$result_num % count($colors)], -20));
        }
    }

    return $chart;
};

$widget_data = $fn_get_data();
?>

<div class="widget widget-staff-staff-with-roles widget-<?= $widget['id'] ?>" data-widget-id="<?= $widget['id'] ?>">
    <div class="row">
        <div class="col-md-12">
            <div class="panel_s">
                <div class="panel-body padding-10">
                    <div class="widget-dragger"></div>

                    <h4 class="pull-left mtop5"><?php echo _l('staff_with_roles'); ?></h4>
                    <div class="clearfix"></div>
                    <div class="row mtop5">
                        <hr class="hr-panel-heading-dashboard">
                    </div>

                    <?php if (count($widget_data['datasets'][0]['data']) > 0) { ?>
                        <div class="relative height-250">
                            <canvas class="chart" height="250" id="chart-<?= $widget['id'] ?>"></canvas>
                        </div>
                    <?php } else { ?>
                        <p class="text-center"><?= _l('not_found') ?></p>
                    <?php } ?>
                </div>
            </div>
        </div>
    </div>
</div>

<script>
    window['perfex_dashboard_widget_<?= $widget['id'] ?>_chart_data'] = <?= json_encode($widget_data) ?>;
</script>
