<?php

defined('BASEPATH') or exit('No direct script access allowed');

class Breaks extends AdminController
{
    public function __construct()
    {
        parent::__construct();
        $this->load->model('Breaks_model'); // Load the breaks model
    }

    public function update_on_break()
{
    if (staff_cant('edit', 'users')) {
        ajax_access_denied();
    }

    if ($this->input->post()) {
        $data = $this->input->post();

        if (!isset($data['id'])) {
            echo json_encode(['success' => false, 'message' => _l('invalid_user_id')]);
            return;
        }

        $user_id = $data['id'];
        unset($data['id']); // Remove ID from data to prevent unwanted updates

        // Fetch the break record to get `start_time`
        $this->db->where('id', $user_id);
        $break = $this->db->get(db_prefix() . 'breaks')->row();

        if ($break) {
            // Calculate `mins_late` if `end_time` and `start_time` are available
            if (!empty($data['end_time']) && !empty($break->start_time)) {
                $start_time = new DateTime($break->start_time);
                $end_time = new DateTime($data['end_time']);
                $interval = $start_time->diff($end_time);
                $data['mins_late'] = $interval->i; // Minutes difference
            } else {
                $data['mins_late'] = 0; // Default to 0 if times are missing
            }
        }

        // Update on_break, end_time, and mins_late
        $this->load->model('Breaks_model');
        $success = $this->Breaks_model->update_on_break_and_mins_late($user_id, $data);

        if ($success) {
            echo json_encode(['success' => true, 'message' => _l('status_updated_successfully')]);
        } else {
            echo json_encode(['success' => false, 'message' => _l('status_update_failed')]);
        }
    } else {
        echo json_encode(['success' => false, 'message' => _l('no_data_received')]);
    }
}

    public function index()
    {
        if (staff_cant('view', 'staff')) {
            access_denied('staff');
        }
        if ($this->input->is_ajax_request()) {
            $this->app->get_table_data('break');
        }
        $data['staff_members'] = $this->staff_model->get('', ['active' => 1]);
        $data['title'] = _l('staff_members');
        $this->load->view('admin/break/manage', $data);
    }

    public function take_break()
    {
        $this->load->view('admin/break/take_break');
    }

    public function create_break()
    {
        if (!is_admin() && get_option('staff_members_create_break') == '0') {
            access_denied('Break Management');
        }

        if ($this->input->post()) {
            $data = $this->input->post();

            if (!$this->input->post('id')) {
                $inline = isset($data['inline']);
                if (isset($data['inline'])) {
                    unset($data['inline']);
                }

                $id = $this->Breaks_model->add_break($data);

                if (!$inline) {
                    if ($id) {
                        set_alert('success', _l('added_successfully', _l('break')));
                    } else {
                        set_alert('danger', _l('add_failed', _l('break')));
                    }
                } else {
                    echo json_encode(['success' => (bool)$id, 'id' => $id]);
                }
            } else {
                $id = $data['id'];
                unset($data['id']);

                $success = $this->Breaks_model->update_break($data, $id);
                if ($success) {
                    set_alert('success', _l('updated_successfully', _l('break')));
                } else {
                    set_alert('danger', _l('update_failed', _l('break')));
                }
            }

            redirect(admin_url('breaks'));
        }
    }

    // public function save_break_late_time()
    // {
    //     $staff_id = $this->input->post('staff_id');
    //     $break_type = $this->input->post('break_type');
    //     $mins_late = $this->input->post('mins_late');

    //     $success = $this->Breaks_model->update_staff_break_with_late_time($staff_id, $break_type, $mins_late);

    //     echo($mins_late);
    //     print_r($mins_late);

    //     if ($success) {
    //         echo json_encode(['success' => true, 'message' => 'Break and late time saved successfully.']);
    //     } else {
    //         echo json_encode(['success' => false, 'message' => 'Failed to save break and late time.']);
    //     }
    // }

    public function start_break()
    {
        $staff_id = get_staff_user_id(); // Assuming a function to get the logged-in user
        $break_type = $this->input->post('break_type');
        $break_minutes = $this->input->post('break_minutes');
    
        $data = [
            'staff_id' => $staff_id,
            'break_type' => $break_type,
            'break_minutes' => $break_minutes,
            'start_time' => date('Y-m-d H:i:s'),
            'on_break' => 1,
            'created_at' => date('Y-m-d H:i:s'),
        ];
    
        $this->db->insert('breaks', $data); // Assuming your table is called `breaks`
        echo json_encode(['success' => true, 'id' => $this->db->insert_id()]);
    }


    public function get_break_status()
{
    $this->load->model('breaks_model');

    $staff_id = get_staff_user_id(); 
    $active_break = $this->breaks_model->check_on_break($staff_id);

    if ($active_break) {
        $response = [
            'success' => true,
            'message' => 'On break',
        ];
    } else {
        $response = [
            'success' => false,
            'message' => 'Not on break',
        ];
    }

    echo json_encode($response);
}
    

    public function end_break()
{
    $break_id = $this->input->post('id'); // The break ID from the client
    $end_time = date('Y-m-d H:i:s');

    // Fetch the break details to calculate late minutes
    $break = $this->db->get_where('breaks', ['id' => $break_id])->row_array();
    if (!$break || $break['on_break'] == 0) {
        echo json_encode(['success' => false, 'message' => 'Break not active or already ended.']);
        return;
    }

    $allowed_end_time = strtotime($break['start_time']) + ($break['break_minutes'] * 60); // Allowed end time
    $actual_end_time = strtotime($end_time);
    $mins_late = max(0, ceil(($actual_end_time - $allowed_end_time) / 60));

    // Update the break record
    $this->db->where('id', $break_id);
    $this->db->update('breaks', [
        'end_time' => $end_time,
        'on_break' => 0,
        'mins_late' => $mins_late,
    ]);

    echo json_encode(['success' => true, 'mins_late' => $mins_late]);
}



    public function save_break_late_time()
{
    $staff_id = $this->input->post('staff_id');
    $break_type = $this->input->post('break_type');
    $mins_late = $this->input->post('mins_late');

    // If mins_late is not provided, default to 0
    if ($mins_late === null) {
        $mins_late = 0;
    }

    $success = $this->Breaks_model->update_staff_break_with_late_time($staff_id, $break_type, $mins_late);

    if ($success) {
        echo json_encode(['success' => true, 'message' => 'Break and late time saved successfully.']);
    } else {
        echo json_encode(['success' => false, 'message' => 'Failed to save break and late time.']);
    }
}


    public function get_all_breaks()
    {
        $breaks = $this->Breaks_model->get_all_breaks();

        if (!empty($breaks)) {
            echo json_encode(['success' => true, 'data' => $breaks]);
        } else {
            echo json_encode(['success' => false, 'message' => 'No breaks available.']);
        }
    }

    public function save_break()
    {
        $data = [
            'staff_id' => $this->input->post('staff_id'),
            'break_type' => $this->input->post('break_type'),
            'break_minutes' => $this->input->post('break_minutes'),
            'start_time' => date('Y-m-d H:i:s')
        ];

        $startTime = strtotime($data['start_time']);
        $data['end_time'] = date('Y-m-d H:i:s', strtotime("+{$data['break_minutes']} minutes", $startTime));

        $success = $this->Breaks_model->create_staff_break($data);

        if ($success) {
            set_alert('success', _l('break started', _l('staff_member')));
        } else {
            set_alert('warning', _l('break cannot start'));
        }
    }

    public function delete()
    {
        if (!is_admin() && is_admin($this->input->post('id'))) {
            die('Busted, you can\'t delete administrators');
        }

        if (staff_can('delete', 'break')) {
            $success = $this->Breaks_model->delete($this->input->post('id'), $this->input->post('transfer_data_to'));
            if ($success) {
                set_alert('success', _l('deleted', _l('break')));
            }
        }

        redirect(admin_url('breaks'));
    }



    public function get_end_time() {
        $staff_id = $this->input->get('staff_id'); // Get the staff_id from the request
        
        // Debugging: log the staff_id to ensure it's passed correctly
        log_message('debug', 'Staff ID received: ' . $staff_id);
    
        // Call the model function to get the end_time
        $end_time = $this->Breaks_model->get_end_time_by_staff($staff_id);
    
        if ($end_time) {
            log_message('debug', 'End Time Retrieved: ' . $end_time); // Log the end_time
            echo json_encode(['success' => true, 'end_time' => $end_time]);
        } else {
            log_message('error', 'Failed to fetch end_time for staff_id: ' . $staff_id); // Log an error
            echo json_encode(['success' => false, 'message' => 'End time not found']);
        }
    }

}
