<?php

use app\services\imap\Imap;
use app\services\RetentionProfileBadges;
use app\services\retentions\retentionsKanban;
use app\services\imap\ConnectionErrorException;
use Ddeboer\Imap\Exception\MailboxDoesNotExistException;

header('Content-Type: text/html; charset=utf-8');
defined('BASEPATH') or exit('No direct script access allowed');

class retentions extends AdminController
{
    public function __construct()
    {
        parent::__construct();
        $this->load->model('retentions_model');
    }

    /* List all retentions */
    public function index($id = '')
    {
        close_setup_menu();

        if (!is_staff_member()) {
            access_denied('retentions');
        }

        $data['switch_kanban'] = true;

        if ($this->session->userdata('retentions_kanban_view') == 'true') {
            $data['switch_kanban'] = false;
            $data['bodyclass']     = 'kan-ban-body';
        }

        $data['staff'] = $this->staff_model->get();
        $data['report_heading'] = $this->input->post('report_heading') == null ? "" :$this->input->post('report_heading');
        $data['statuses'] = $this->retentions_model->get_status();
        $data['sources']  = $this->retentions_model->get_source();
        $data['title']    = _l('retentions');
        $data['table'] = App_table::find('retentions');
        $data['countries'] = $this->input->post('countries') == null ? [] :$this->input->post('countries');
        $data['hide_columns'] = $this->input->post('hide_columns') == null ? [] :$this->input->post('hide_columns');
        $data['date_by'] = $this->input->post('date_by') == null ? 'dateadded' :$this->input->post('date_by');
        $data['report_months'] = $this->input->post('report_months') == null ? 'this_month' :$this->input->post('report_months');
        // in case accesed the url retentions/index/ directly with id - used in search
        $data['retentionid']   = $id;
        $this->load->view('admin/retentions/manage_retentions', $data);
    }

    public function table()
    {
        if (!is_staff_member()) {
            ajax_access_denied();
        }

        App_table::find('retentions')->output();
    }

    public function kanban()
    {
        if (!is_staff_member()) {
            ajax_access_denied();
        }

        $data['statuses']      = $this->retentions_model->get_status();
        $data['base_currency'] = get_base_currency();
        $data['summary']       = get_retentions_summary();

        echo $this->load->view('admin/retentions/kan-ban', $data, true);
    }

    /* Add or update retention */
    public function retention($id = '')
    {
        if (!is_staff_member() || ($id != '' && !$this->retentions_model->staff_can_access_retention($id))) {
            ajax_access_denied();
        }

        if ($this->input->post()) {
            if ($id == '') {
                $id      = $this->retentions_model->add($this->input->post());
                $message = $id ? _l('added_successfully', _l('retention')) : '';

                echo json_encode([
                    'success'  => $id ? true : false,
                    'id'       => $id,
                    'message'  => $message,
                    'retentionView' => $id ? $this->_get_retention_data($id) : [],
                ]);
            } else {
                $emailOriginal   = $this->db->select('email')->where('id', $id)->get(db_prefix() . 'retentions')->row()->email;
                $proposalWarning = false;
                $message         = '';
                $success         = $this->retentions_model->update($this->input->post(), $id);

                if ($success) {
                    $emailNow = $this->db->select('email')->where('id', $id)->get(db_prefix() . 'retentions')->row()->email;

                    $proposalWarning = (total_rows(db_prefix() . 'proposals', [
                        'rel_type' => 'retention',
                        'rel_id'   => $id, ]) > 0 && ($emailOriginal != $emailNow) && $emailNow != '') ? true : false;

                    $message = _l('updated_successfully', _l('retention'));
                }
                echo json_encode([
                    'success'          => $success,
                    'message'          => $message,
                    'id'               => $id,
                    'proposal_warning' => $proposalWarning,
                    'retentionView'         => $this->_get_retention_data($id),
                ]);
            }
            die;
        }

        echo json_encode([
            'retentionView' => $this->_get_retention_data($id),
        ]);
    }

    private function _get_retention_data($id = '')
    {
        $reminder_data         = '';
        $data['retention_locked']   = false;
        $data['openEdit']      = $this->input->get('edit') ? true : false;
        $data['members']       = $this->staff_model->get('', ['is_not_staff' => 0, 'active' => 1]);
        $data['status_id']     = $this->input->get('status_id') ? $this->input->get('status_id') : get_option('retentions_default_status');
        $data['base_currency'] = get_base_currency();

        if (is_numeric($id)) {
            $retentionWhere = (staff_can('view',  'retentions') ? [] : '(assigned = ' . get_staff_user_id() . ' OR addedfrom=' . get_staff_user_id() . ' OR is_public=1)');
       
            $retention = $this->retentions_model->get($id, $retentionWhere);

            if (!$retention) {
                header('HTTP/1.0 404 Not Found');
                echo _l('retention_not_found');
                die;
            }

            if (total_rows(db_prefix() . 'clients', ['retentionid' => $id ]) > 0) {
                $data['retention_locked'] = ((!is_admin() && get_option('retention_lock_after_convert_to_customer') == 1) ? true : false);
            }

            $reminder_data = $this->load->view('admin/includes/modals/reminder', [
                    'id'             => $retention->id,
                    'name'           => 'retention',
                    'members'        => $data['members'],
                    'reminder_title' => _l('retention_set_reminder_title'),
                ], true);
            $data['reminders']          = $this->retentions_model->get_reminders($retention->id);

            $data['retention']          = $retention;
            $data['retentionidfromdb']          = $retention->id;
            $data['mail_activity'] = $this->retentions_model->get_mail_activity($id);
            $data['notes']         = $this->misc_model->get_notes($id, 'retention');
            $data['activity_log']  = $this->retentions_model->get_retention_activity_log($id);

            if (is_gdpr() && get_option('gdpr_enable_consent_for_retentions') == '1') {
                $this->load->model('gdpr_model');
                $data['purposes'] = $this->gdpr_model->get_consent_purposes($retention->id, 'retention');
                $data['consents'] = $this->gdpr_model->get_consents(['retention_id' => $retention->id]);
            }

            $retentionProfileBadges         = new RetentionProfileBadges($id);
            $data['total_reminders']   = $retentionProfileBadges->getCount('reminders');
            $data['total_notes']       = $retentionProfileBadges->getCount('notes');
            $data['total_attachments'] = $retentionProfileBadges->getCount('attachments');
            $data['total_tasks']       = $retentionProfileBadges->getCount('tasks');
            $data['total_proposals']   = $retentionProfileBadges->getCount('proposals');
        }


        $data['statuses'] = $this->retentions_model->get_status();
        $data['sources']  = $this->retentions_model->get_source();
        $this->db->select('*');
            $retentionids = $this->db->get('retentions');
            $retentionidsarray = $retentionids->result_array();

            // Extract Department_IDs from the result
            $retention_ids = array_map(function ($item) {
                return $item['id'];
            }, $retentionidsarray);
        $data['retentionids']  = $retention_ids;
        $data = hooks()->apply_filters('retention_view_data', $data);

        return [
            'data'          => $this->load->view('admin/retentions/retention', $data, true),
            'reminder_data' => $reminder_data,
        ];
    }

    public function retentions_kanban_load_more()
    {
        if (!is_staff_member()) {
            ajax_access_denied();
        }

        $status = $this->input->get('status');
        $page   = $this->input->get('page');

        $this->db->where('id', $status);
        $status = $this->db->get(db_prefix() . 'retentions_status')->row_array();

        $retentions = (new retentionsKanban($status['id']))
        ->search($this->input->get('search'))
        ->sortBy(
            $this->input->get('sort_by'),
            $this->input->get('sort')
        )
        ->page($page)->get();

        foreach ($retentions as $retention) {
            $this->load->view('admin/retentions/_kan_ban_card', [
                'retention'   => $retention,
                'status' => $status,
            ]);
        }
    }

    public function switch_kanban($set = 0)
    {
        if ($set == 1) {
            $set = 'true';
        } else {
            $set = 'false';
        }
        $this->session->set_userdata([
            'retentions_kanban_view' => $set,
        ]);
        redirect(previous_url() ?: $_SERVER['HTTP_REFERER']);
    }

    public function export($id)
    {
        if (is_admin()) {
            $this->load->library('gdpr/gdpr_retention');
            $this->gdpr_retention->export($id);
        }
    }

    /* Delete retention from database */
    public function delete($id)
    {
        if (!$id) {
            redirect(admin_url('retentions'));
        }

        if (staff_cant('delete', 'retentions')) {
            access_denied('Delete retention');
        }

        $response = $this->retentions_model->delete($id);
        if (is_array($response) && isset($response['referenced'])) {
            set_alert('warning', _l('is_referenced', _l('retention_lowercase')));
        } elseif ($response === true) {
            set_alert('success', _l('deleted', _l('retention')));
        } else {
            set_alert('warning', _l('problem_deleting', _l('retention_lowercase')));
        }

        $ref = $_SERVER['HTTP_REFERER'];

        // if user access retentions/inded/ID to prevent redirecting on the same url because will throw 404
        if (!$ref || strpos($ref, 'index/' . $id) !== false) {
            redirect(admin_url('retentions'));
        }

        redirect($ref);
    }

    public function mark_as_lost($id)
    {
        if (!is_staff_member() || !$this->retentions_model->staff_can_access_retention($id)) {
            ajax_access_denied();
        }
        $message = '';
        $success = $this->retentions_model->mark_as_lost($id);
        if ($success) {
            $message = _l('retention_marked_as_lost');
        }
        echo json_encode([
            'success'  => $success,
            'message'  => $message,
            'retentionView' => $this->_get_retention_data($id),
            'id'       => $id,
        ]);
    }

    public function retention_reload($id)
    {
        if (!is_staff_member() || !$this->retentions_model->staff_can_access_retention($id)) {
            ajax_access_denied();
        }
    
       
        // echo json_encode([

           
        //     'retentionView' => $this->_get_retention_data($id),
        //     'id'       => $id,
        // ]);

        echo json_encode(['retentionView' => $this->_get_retention_data($id), 'id' => $id]);

    }



    public function unmark_as_lost($id)
    {
        if (!is_staff_member() || !$this->retentions_model->staff_can_access_retention($id)) {
            ajax_access_denied();
        }
        $message = '';
        $success = $this->retentions_model->unmark_as_lost($id);
        if ($success) {
            $message = _l('retention_unmarked_as_lost');
        }
        echo json_encode([
            'success'  => $success,
            'message'  => $message,
            'retentionView' => $this->_get_retention_data($id),
            'id'       => $id,
        ]);
    }

    public function mark_as_junk($id)
    {
        if (!is_staff_member() || !$this->retentions_model->staff_can_access_retention($id)) {
            ajax_access_denied();
        }
        $message = '';
        $success = $this->retentions_model->mark_as_junk($id);
        if ($success) {
            $message = _l('retention_marked_as_junk');
        }
        echo json_encode([
            'success'  => $success,
            'message'  => $message,
            'retentionView' => $this->_get_retention_data($id),
            'id'       => $id,
        ]);
    }

    public function unmark_as_junk($id)
    {
        if (!is_staff_member() || !$this->retentions_model->staff_can_access_retention($id)) {
            ajax_access_denied();
        }
        $message = '';
        $success = $this->retentions_model->unmark_as_junk($id);
        if ($success) {
            $message = _l('retention_unmarked_as_junk');
        }
        echo json_encode([
            'success'  => $success,
            'message'  => $message,
            'retentionView' => $this->_get_retention_data($id),
            'id'       => $id,
        ]);
    }

    public function add_activity()
    {
        $retentionid = $this->input->post('retentionid');
        if (!is_staff_member() || !$this->retentions_model->staff_can_access_retention($retentionid)) {
            ajax_access_denied();
        }
        if ($this->input->post()) {
            $message = $this->input->post('activity');
            $aId     = $this->retentions_model->log_retention_activity($retentionid, $message);
            if ($aId) {
                $this->db->where('id', $aId);
                $this->db->update(db_prefix() . 'retention_activity_log', ['custom_activity' => 1]);
            }
            echo json_encode(['retentionView' => $this->_get_retention_data($retentionid), 'id' => $retentionid]);
        }
    }

    public function get_convert_data($id)
    {
        if (!is_staff_member() || !$this->retentions_model->staff_can_access_retention($id)) {
            ajax_access_denied();
        }
        if (is_gdpr() && get_option('gdpr_enable_consent_for_contacts') == '1') {
            $this->load->model('gdpr_model');
            $data['purposes'] = $this->gdpr_model->get_consent_purposes($id, 'retention');
        }
        $data['retention'] = $this->retentions_model->get($id);
        $this->load->view('admin/retentions/convert_to_customer', $data);
    }

    /**
     * Convert retention to client
     * @since  version 1.0.1
     * @return mixed
     */
    public function convert_to_customer()
    {
        if (!is_staff_member()) {
            access_denied('retention Convert to Customer');
        }

        if ($this->input->post()) {
            $default_country  = get_option('customer_default_country');
            $data             = $this->input->post();
            $data['password'] = $this->input->post('password', false);

            $original_retention_email = $data['original_retention_email'];
            unset($data['original_retention_email']);

            if (isset($data['transfer_notes'])) {
                $notes = $this->misc_model->get_notes($data['retentionid'], 'retention');
                unset($data['transfer_notes']);
            }

            if (isset($data['transfer_consent'])) {
                $this->load->model('gdpr_model');
                $consents = $this->gdpr_model->get_consents(['retention_id' => $data['retentionid']]);
                unset($data['transfer_consent']);
            }

            if (isset($data['merge_db_fields'])) {
                $merge_db_fields = $data['merge_db_fields'];
                unset($data['merge_db_fields']);
            }

            if (isset($data['merge_db_contact_fields'])) {
                $merge_db_contact_fields = $data['merge_db_contact_fields'];
                unset($data['merge_db_contact_fields']);
            }

            if (isset($data['include_retentions_custom_fields'])) {
                $include_retentions_custom_fields = $data['include_retentions_custom_fields'];
                unset($data['include_retentions_custom_fields']);
            }

            if ($data['country'] == '' && $default_country != '') {
                $data['country'] = $default_country;
            }

            $data['billing_street']  = $data['address'];
            $data['billing_city']    = $data['city'];
            $data['billing_state']   = $data['state'];
            $data['billing_zip']     = $data['zip'];
            $data['billing_country'] = $data['country'];

            $data['is_primary'] = 1;
            $id                 = $this->clients_model->add($data, true);
            if ($id) {
                $primary_contact_id = get_primary_contact_user_id($id);

                if (isset($notes)) {
                    foreach ($notes as $note) {
                        $this->db->insert(db_prefix() . 'notes', [
                            'rel_id'         => $id,
                            'rel_type'       => 'customer',
                            'dateadded'      => $note['dateadded'],
                            'addedfrom'      => $note['addedfrom'],
                            'description'    => $note['description'],
                            'date_contacted' => $note['date_contacted'],
                            ]);
                    }
                }
                if (isset($consents)) {
                    foreach ($consents as $consent) {
                        unset($consent['id']);
                        unset($consent['purpose_name']);
                        $consent['retention_id']    = 0;
                        $consent['contact_id'] = $primary_contact_id;
                        $this->gdpr_model->add_consent($consent);
                    }
                }
                if (staff_cant('view', 'customers') && get_option('auto_assign_customer_admin_after_retention_convert') == 1) {
                    $this->db->insert(db_prefix() . 'customer_admins', [
                        'date_assigned' => date('Y-m-d H:i:s'),
                        'customer_id'   => $id,
                        'staff_id'      => get_staff_user_id(),
                    ]);
                }
                $this->retentions_model->log_retention_activity($data['retentionid'], 'not_retention_activity_converted', false, serialize([
                    get_staff_full_name(),
                ]));
                $default_status = $this->retentions_model->get_status('', [
                    'isdefault' => 1,
                ]);
                $this->db->where('id', $data['retentionid']);
                $this->db->update(db_prefix() . 'retentions', [
                    'date_converted' => date('Y-m-d H:i:s'),
                    'status'         => $default_status[0]['id'],
                    'junk'           => 0,
                    'lost'           => 0,
                ]);
                // Check if retention email is different then client email
                $contact = $this->clients_model->get_contact(get_primary_contact_user_id($id));
                if ($contact->email != $original_retention_email) {
                    if ($original_retention_email != '') {
                        $this->retentions_model->log_retention_activity($data['retentionid'], 'not_retention_activity_converted_email', false, serialize([
                            $original_retention_email,
                            $contact->email,
                        ]));
                    }
                }
                if (isset($include_retentions_custom_fields)) {
                    foreach ($include_retentions_custom_fields as $fieldid => $value) {
                        // checked don't merge
                        if ($value == 5) {
                            continue;
                        }
                        // get the value of this retentions custom fiel
                        $this->db->where('relid', $data['retentionid']);
                        $this->db->where('fieldto', 'retentions');
                        $this->db->where('fieldid', $fieldid);
                        $retention_custom_field_value = $this->db->get(db_prefix() . 'customfieldsvalues')->row()->value;
                        // Is custom field for contact ot customer
                        if ($value == 1 || $value == 4) {
                            if ($value == 4) {
                                $field_to = 'contacts';
                            } else {
                                $field_to = 'customers';
                            }
                            $this->db->where('id', $fieldid);
                            $field = $this->db->get(db_prefix() . 'customfields')->row();
                            // check if this field exists for custom fields
                            $this->db->where('fieldto', $field_to);
                            $this->db->where('name', $field->name);
                            $exists               = $this->db->get(db_prefix() . 'customfields')->row();
                            $copy_custom_field_id = null;
                            if ($exists) {
                                $copy_custom_field_id = $exists->id;
                            } else {
                                // there is no name with the same custom field for retentions at the custom side create the custom field now
                                $this->db->insert(db_prefix() . 'customfields', [
                                    'fieldto'        => $field_to,
                                    'name'           => $field->name,
                                    'required'       => $field->required,
                                    'type'           => $field->type,
                                    'options'        => $field->options,
                                    'display_inline' => $field->display_inline,
                                    'field_order'    => $field->field_order,
                                    'slug'           => slug_it($field_to . '_' . $field->name, [
                                        'separator' => '_',
                                    ]),
                                    'active'        => $field->active,
                                    'only_admin'    => $field->only_admin,
                                    'show_on_table' => $field->show_on_table,
                                    'bs_column'     => $field->bs_column,
                                ]);
                                $new_customer_field_id = $this->db->insert_id();
                                if ($new_customer_field_id) {
                                    $copy_custom_field_id = $new_customer_field_id;
                                }
                            }
                            if ($copy_custom_field_id != null) {
                                $insert_to_custom_field_id = $id;
                                if ($value == 4) {
                                    $insert_to_custom_field_id = get_primary_contact_user_id($id);
                                }
                                $this->db->insert(db_prefix() . 'customfieldsvalues', [
                                    'relid'   => $insert_to_custom_field_id,
                                    'fieldid' => $copy_custom_field_id,
                                    'fieldto' => $field_to,
                                    'value'   => $retention_custom_field_value,
                                ]);
                            }
                        } elseif ($value == 2) {
                            if (isset($merge_db_fields)) {
                                $db_field = $merge_db_fields[$fieldid];
                                // in case user don't select anything from the db fields
                                if ($db_field == '') {
                                    continue;
                                }
                                if ($db_field == 'country' || $db_field == 'shipping_country' || $db_field == 'billing_country') {
                                    $this->db->where('iso2', $retention_custom_field_value);
                                    $this->db->or_where('short_name', $retention_custom_field_value);
                                    $this->db->or_like('long_name', $retention_custom_field_value);
                                    $country = $this->db->get(db_prefix() . 'countries')->row();
                                    if ($country) {
                                        $retention_custom_field_value = $country->country_id;
                                    } else {
                                        $retention_custom_field_value = 0;
                                    }
                                }
                                $this->db->where('userid', $id);
                                $this->db->update(db_prefix() . 'clients', [
                                    $db_field => $retention_custom_field_value,
                                ]);
                            }
                        } elseif ($value == 3) {
                            if (isset($merge_db_contact_fields)) {
                                $db_field = $merge_db_contact_fields[$fieldid];
                                if ($db_field == '') {
                                    continue;
                                }
                                $this->db->where('id', $primary_contact_id);
                                $this->db->update(db_prefix() . 'contacts', [
                                    $db_field => $retention_custom_field_value,
                                ]);
                            }
                        }
                    }
                }
                // set the retention to status client in case is not status client
                $this->db->where('isdefault', 1);
                $status_client_id = $this->db->get(db_prefix() . 'retentions_status')->row()->id;
                $this->db->where('id', $data['retentionid']);
                $this->db->update(db_prefix() . 'retentions', [
                    'status' => $status_client_id,
                ]);

                set_alert('success', _l('retention_to_client_base_converted_success'));

                if (is_gdpr() && get_option('gdpr_after_retention_converted_delete') == '1') {
                    // When retention is deleted
                    // move all proposals to the actual customer record
                    $this->db->where('rel_id', $data['retentionid']);
                    $this->db->where('rel_type', 'retention');
                    $this->db->update('proposals', [
                        'rel_id'   => $id,
                        'rel_type' => 'customer',
                    ]);

                    $this->retentions_model->delete($data['retentionid']);

                    $this->db->where('userid', $id);
                    $this->db->update(db_prefix() . 'clients', ['retentionid' => null]);
                }

                log_activity('Created retention Client Profile [retentionID: ' . $data['retentionid'] . ', ClientID: ' . $id . ']');
                hooks()->do_action('retention_converted_to_customer', ['retention_id' => $data['retentionid'], 'customer_id' => $id]);
                redirect(admin_url('clients/client/' . $id));
            }
        }
    }

    /* Used in kanban when dragging and mark as */
    public function update_retention_status()
    {
        if ($this->input->post() && $this->input->is_ajax_request()) {
            
           $id = $this->retentions_model->update_retention_status($this->input->post());
            if ($id) {
                echo json_encode([
                    'success'  => true,
                    'message'  => 'Updated retention',
                ]);
            }
        }
    }

    public function update_retention_description($id)
    {
        if (staff_can('edit',  'retentions') || staff_can('view_own',  'retentions')) {
            $data = hooks()->apply_filters('before_update_retention', [
                'description' => html_purify($this->input->post('description', false)),
            ], $id);

            $this->db->where('id', $id);
            $insert = $this->db->update(db_prefix() . 'retentions', $data);
            echo json_encode(['retentionView' => $lastFile ? $this->_get_retention_data($id) : [], 'id' => $id]);


            hooks()->do_action('after_update_retention', $id);
        }
    }

    public function update_retention_name($id)
    {
        if (staff_can('edit', 'retentions') || staff_can('view_own', 'retentions')) {
            // Retrieve and clean the name from POST data
            $name = $this->input->post('name', false);
            $data = hooks()->apply_filters('before_update_retention', [
                'name' => html_purify($name),
            ], $id);

            // Update the retention name in the database
            $this->db->where('id', $id);
            $success = $this->db->update(db_prefix() . 'retentions', $data);

            // Trigger hooks after updating
            hooks()->do_action('after_update_retention', $id);

            // Send JSON response back to the frontend
            if ($success) {
                echo json_encode([
                    'success' => true,
                    'message' => 'Updated retention Name',
                ]);
            } else {
                echo json_encode([
                    'success' => false,
                    'message' => 'Failed to update retention name',
                ]);
            }
            exit; // End the script after sending the response
        } else {
            // Send error response if the user doesn't have permission
            echo json_encode([
                'success' => false,
                'message' => 'Unauthorized',
            ]);
            exit;
        }
    }

    public function update_retention_email($id)
    {
        if (staff_can('edit', 'retentions') || staff_can('view_own', 'retentions')) {
            // Retrieve and clean the email from POST data
            $email = $this->input->post('email', false);
            $data = hooks()->apply_filters('before_update_retention', [
                'email' => html_purify($email),
            ], $id);

            // Update the retention email in the database
            $this->db->where('id', $id);
            $success = $this->db->update(db_prefix() . 'retentions', $data);

            // Trigger hooks after updating
            hooks()->do_action('after_update_retention', $id);

            // Send JSON response back to the frontend
            if ($success) {
                echo json_encode([
                    'success' => true,
                    'message' => 'Updated retention Email',
                ]);
            } else {
                echo json_encode([
                    'success' => false,
                    'message' => 'Failed to update retention email',
                ]);
            }
            exit; // End the script after sending the response
        } else {
            // Send error response if the user doesn't have permission
            echo json_encode([
                'success' => false,
                'message' => 'Unauthorized',
            ]);
            exit;
        }
    }

    public function update_retention_phonenumber($id)
    {
        if (staff_can('edit', 'retentions') || staff_can('view_own', 'retentions')) {
            // Retrieve and clean the phonenumber from POST data
            $phonenumber = $this->input->post('phonenumber', false);
            $data = hooks()->apply_filters('before_update_retention', [
                'phonenumber' => html_purify($phonenumber),
            ], $id);

            // Update the retention phonenumber in the database
            $this->db->where('id', $id);
            $success = $this->db->update(db_prefix() . 'retentions', $data);

            // Trigger hooks after updating
            hooks()->do_action('after_update_retention', $id);

            // Send JSON response back to the frontend
            if ($success) {
                echo json_encode([
                    'success' => true,
                    'message' => 'Updated retention PhoneNumber',
                ]);
            } else {
                echo json_encode([
                    'success' => false,
                    'message' => 'Failed to update retention phonenumber',
                ]);
            }
            exit; // End the script after sending the response
        } else {
            // Send error response if the user doesn't have permission
            echo json_encode([
                'success' => false,
                'message' => 'Unauthorized',
            ]);
            exit;
        }
    }

    public function update_retention_value($id)
    {
        if (staff_can('edit', 'retentions') || staff_can('view_own', 'retentions')) {
            // Retrieve and clean the value from POST data
            $value = $this->input->post('value', false);
            $data = hooks()->apply_filters('before_update_retention', [
                'retention_value' => html_purify($value),
            ], $id);

            // Update the retention value in the database
            $this->db->where('id', $id);
            $success = $this->db->update(db_prefix() . 'retentions', $data);

            // Trigger hooks after updating
            hooks()->do_action('after_update_retention', $id);

            // Send JSON response back to the frontend
            if ($success) {
                echo json_encode([
                    'success' => true,
                    'message' => 'Updated retention Value',
                ]);
            } else {
                echo json_encode([
                    'success' => false,
                    'message' => 'Failed to update retention value',
                ]);
            }
            exit; // End the script after sending the response
        } else {
            // Send error response if the user doesn't have permission
            echo json_encode([
                'success' => false,
                'message' => 'Unauthorized',
            ]);
            exit;
        }
    }




    // public function update_retention_description($id)
    // {
    //     if (staff_can('edit', 'retentions')) {
    //         // Retrieve and clean the description from POST data
    //         $description = $this->input->post('description', false);
    //         $data = hooks()->apply_filters('before_update_retention', [
    //             'description' => html_purify($description),
    //         ], $id);

    //         // Update the retention description in the database
    //         $this->db->where('id', $id);
    //         $success = $this->db->update(db_prefix() . 'retentions', $data);

    //         // Trigger hooks after updating
    //         hooks()->do_action('after_update_retention', $id);

    //         // Send JSON response back to the frontend
    //         if ($success) {
    //             echo json_encode([
    //                 'success' => true,
    //                 'message' => 'Updated retention',
    //             ]);
    //         } else {
    //             echo json_encode([
    //                 'success' => false,
    //                 'message' => 'Failed to update retention',
    //             ]);
    //         }
    //         exit; // Make sure to end the script after sending the response
    //     } else {
    //         // Send error response if the user doesn't have permission
    //         echo json_encode([
    //             'success' => false,
    //             'message' => 'Unauthorized',
    //         ]);
    //         exit;
    //     }
    // }


    

    public function update_status_order()
    {
        if ($post_data = $this->input->post()) {
            $this->retentions_model->update_status_order($post_data);
        }
    }

    public function add_retention_attachment()
    {
        $id       = $this->input->post('id');
        $lastFile = $this->input->post('last_file');

        if (!is_staff_member() || !$this->retentions_model->staff_can_access_retention($id)) {
            ajax_access_denied();
        }

        handle_retention_attachments($id);
        echo json_encode(['retentionView' => $lastFile ? $this->_get_retention_data($id) : [], 'id' => $id]);
    }

    public function add_external_attachment()
    {
        if ($this->input->post()) {
            $this->retentions_model->add_attachment_to_database(
                $this->input->post('retention_id'),
                $this->input->post('files'),
                $this->input->post('external')
            );
        }
    }

    public function delete_attachment($id, $retention_id)
    {
        if (!is_staff_member() || !$this->retentions_model->staff_can_access_retention($retention_id)) {
            ajax_access_denied();
        }
        echo json_encode([
            'success'  => $this->retentions_model->delete_retention_attachment($id),
            'retentionView' => $this->_get_retention_data($retention_id),
            'id'       => $retention_id,
        ]);
    }

    public function delete_note($id, $retention_id)
    {
        if (!is_staff_member() || !$this->retentions_model->staff_can_access_retention($retention_id)) {
            ajax_access_denied();
        }
        echo json_encode([
            'success'  => $this->misc_model->delete_note($id),
            'retentionView' => $this->_get_retention_data($retention_id),
            'id'       => $retention_id,
        ]);
    }

    public function update_all_proposal_emails_linked_to_retention($id)
    {
        $success = false;
        $email   = '';
        if ($this->input->post('update')) {
            $this->load->model('proposals_model');

            $this->db->select('email');
            $this->db->where('id', $id);
            $email = $this->db->get(db_prefix() . 'retentions')->row()->email;

            $proposals = $this->proposals_model->get('', [
                'rel_type' => 'retention',
                'rel_id'   => $id,
            ]);
            $affected_rows = 0;

            foreach ($proposals as $proposal) {
                $this->db->where('id', $proposal['id']);
                $this->db->update(db_prefix() . 'proposals', [
                    'email' => $email,
                ]);
                if ($this->db->affected_rows() > 0) {
                    $affected_rows++;
                }
            }

            if ($affected_rows > 0) {
                $success = true;
            }
        }

        echo json_encode([
            'success' => $success,
            'message' => _l('proposals_emails_updated', [
                _l('retention_lowercase'),
                $email,
            ]),
        ]);
    }

    public function save_form_data()
    {
        $data = $this->input->post();

        // form data should be always sent to the request and never should be empty
        // this code is added to prevent losing the old form in case any errors
        if (!isset($data['formData']) || isset($data['formData']) && !$data['formData']) {
            echo json_encode([
                'success' => false,
            ]);
            die;
        }

        // If user paste with styling eq from some editor word and the Codeigniter XSS feature remove and apply xss=remove, may break the json.
        $data['formData'] = preg_replace('/=\\\\/m', "=''", $data['formData']);

        $this->db->where('id', $data['id']);
        $this->db->update(db_prefix() . 'web_to_retention', [
            'form_data' => $data['formData'],
        ]);
        if ($this->db->affected_rows() > 0) {
            echo json_encode([
                'success' => true,
                'message' => _l('updated_successfully', _l('web_to_retention_form')),
            ]);
        } else {
            echo json_encode([
                'success' => false,
            ]);
        }
    }

    public function form($id = '')
    {
        if (!is_admin()) {
            access_denied('Web To retention Access');
        }
        if ($this->input->post()) {
            if ($id == '') {
                $data = $this->input->post();
                $id   = $this->retentions_model->add_form($data);
                if ($id) {
                    set_alert('success', _l('added_successfully', _l('web_to_retention_form')));
                    redirect(admin_url('retentions/form/' . $id));
                }
            } else {
                $success = $this->retentions_model->update_form($id, $this->input->post());
                if ($success) {
                    set_alert('success', _l('updated_successfully', _l('web_to_retention_form')));
                }
                redirect(admin_url('retentions/form/' . $id));
            }
        }

        $data['formData'] = [];
        $custom_fields    = get_custom_fields('retentions', 'type != "link"');

        $cfields       = format_external_form_custom_fields($custom_fields);
        $data['title'] = _l('web_to_retention');

        if ($id != '') {
            $data['form'] = $this->retentions_model->get_form([
                'id' => $id,
            ]);
            $data['title']    = $data['form']->name . ' - ' . _l('web_to_retention_form');
            $data['formData'] = $data['form']->form_data;
        }

        $this->load->model('roles_model');
        $data['roles']    = $this->roles_model->get();
        $data['sources']  = $this->retentions_model->get_source();
        $data['statuses'] = $this->retentions_model->get_status();

        $data['members'] = $this->staff_model->get('', [
            'active'       => 1,
            'is_not_staff' => 0,
        ]);

        $data['languages'] = $this->app->get_available_languages();
        $data['cfields']   = $cfields;

        $db_fields = [];
        $fields    = [
            'name',
            'title',
            'email',
            'phonenumber',
            'retention_value',
            'company',
            'address',
            'city',
            'state',
            'country',
            'zip',
            'description',
            'website',
        ];

        $fields = hooks()->apply_filters('retention_form_available_database_fields', $fields);

        $className = 'form-control';

        foreach ($fields as $f) {
            $_field_object = new stdClass();
            $type          = 'text';
            $subtype       = '';
            if ($f == 'email') {
                $subtype = 'email';
            } elseif ($f == 'description' || $f == 'address') {
                $type = 'textarea';
            } elseif ($f == 'country') {
                $type = 'select';
            }

            if ($f == 'name') {
                $label = _l('retention_add_edit_name');
            } elseif ($f == 'email') {
                $label = _l('retention_add_edit_email');
            } elseif ($f == 'phonenumber') {
                $label = _l('retention_add_edit_phonenumber');
            } elseif ($f == 'retention_value') {
                $label = _l('retention_add_edit_retention_value');
                $type  = 'number';
            } else {
                $label = _l('retention_' . $f);
            }

            $field_array = [
                'subtype'   => $subtype,
                'type'      => $type,
                'label'     => $label,
                'className' => $className,
                'name'      => $f,
            ];

            if ($f == 'country') {
                $field_array['values'] = [];

                $field_array['values'][] = [
                    'label'    => '',
                    'value'    => '',
                    'selected' => false,
                ];

                $countries = get_all_countries();
                foreach ($countries as $country) {
                    $selected = false;
                    if (get_option('customer_default_country') == $country['country_id']) {
                        $selected = true;
                    }
                    array_push($field_array['values'], [
                        'label'    => $country['short_name'],
                        'value'    => (int) $country['country_id'],
                        'selected' => $selected,
                    ]);
                }
            }

            if ($f == 'name') {
                $field_array['required'] = true;
            }

            $_field_object->label    = $label;
            $_field_object->name     = $f;
            $_field_object->fields   = [];
            $_field_object->fields[] = $field_array;
            $db_fields[]             = $_field_object;
        }
        $data['bodyclass'] = 'web-to-retention-form';
        $data['db_fields'] = $db_fields;
        $this->load->view('admin/retentions/formbuilder', $data);
    }

    public function forms($id = '')
    {
        if (!is_admin()) {
            access_denied('Web To retention Access');
        }

        if ($this->input->is_ajax_request()) {
            $this->app->get_table_data('web_to_retention');
        }

        $data['title'] = _l('web_to_retention');
        $this->load->view('admin/retentions/forms', $data);
    }

    public function delete_form($id)
    {
        if (!is_admin()) {
            access_denied('Web To retention Access');
        }

        $success = $this->retentions_model->delete_form($id);
        if ($success) {
            set_alert('success', _l('deleted', _l('web_to_retention_form')));
        }

        redirect(admin_url('retentions/forms'));
    }

    // Sources
    /* Manage retentions sources */
    public function sources()
    {
        if (!is_admin()) {
            access_denied('retentions Sources');
        }
        $data['sources'] = $this->retentions_model->get_source();
        $data['title']   = 'retentions sources';
        $this->load->view('admin/retentions/manage_sources', $data);
    }

    /* Add or update retentions sources */
    public function source()
    {
        if (!is_admin() && get_option('staff_members_create_inline_retention_source') == '0') {
            access_denied('retentions Sources');
        }
        if ($this->input->post()) {
            $data = $this->input->post();
            if (!$this->input->post('id')) {
                $inline = isset($data['inline']);
                if (isset($data['inline'])) {
                    unset($data['inline']);
                }

                $id = $this->retentions_model->add_source($data);

                if (!$inline) {
                    if ($id) {
                        set_alert('success', _l('added_successfully', _l('retention_source')));
                    }
                } else {
                    echo json_encode(['success' => $id ? true : false, 'id' => $id]);
                }
            } else {
                $id = $data['id'];
                unset($data['id']);
                $success = $this->retentions_model->update_source($data, $id);
                if ($success) {
                    set_alert('success', _l('updated_successfully', _l('retention_source')));
                }
            }
        }
    }

    /* Delete retentions source */
    public function delete_source($id)
    {
        if (!is_admin()) {
            access_denied('Delete retention Source');
        }
        if (!$id) {
            redirect(admin_url('retentions/sources'));
        }
        $response = $this->retentions_model->delete_source($id);
        if (is_array($response) && isset($response['referenced'])) {
            set_alert('warning', _l('is_referenced', _l('retention_source_lowercase')));
        } elseif ($response == true) {
            set_alert('success', _l('deleted', _l('retention_source')));
        } else {
            set_alert('warning', _l('problem_deleting', _l('retention_source_lowercase')));
        }
        redirect(admin_url('retentions/sources'));
    }

    // Statuses
    /* View retentions statuses */
    public function statuses()
    {
        if (!is_admin()) {
            access_denied('retentions Statuses');
        }
        $data['statuses'] = $this->retentions_model->get_status();
        $data['title']    = 'retentions statuses';
        $this->load->view('admin/retentions/manage_statuses', $data);
    }

    /* Add or update retentions status */
    public function status()
    {
        if (!is_admin() && get_option('staff_members_create_inline_retention_status') == '0') {
            access_denied('retentions Statuses');
        }
        if ($this->input->post()) {
            $data = $this->input->post();
            if (!$this->input->post('id')) {
                $inline = isset($data['inline']);
                if (isset($data['inline'])) {
                    unset($data['inline']);
                }
                $id = $this->retentions_model->add_status($data);
                if (!$inline) {
                    if ($id) {
                        set_alert('success', _l('added_successfully', _l('retention_status')));
                    }
                } else {
                    echo json_encode(['success' => $id ? true : false, 'id' => $id]);
                }
            } else {
                $id = $data['id'];
                unset($data['id']);
                $success = $this->retentions_model->update_status($data, $id);
                if ($success) {
                    set_alert('success', _l('updated_successfully', _l('retention_status')));
                }
            }
        }
    }

    /* Delete retentions status from databae */
    public function delete_status($id)
    {
        if (!is_admin()) {
            access_denied('retentions Statuses');
        }
        if (!$id) {
            redirect(admin_url('retentions/statuses'));
        }
        $response = $this->retentions_model->delete_status($id);
        if (is_array($response) && isset($response['referenced'])) {
            set_alert('warning', _l('is_referenced', _l('retention_status_lowercase')));
        } elseif ($response == true) {
            set_alert('success', _l('deleted', _l('retention_status')));
        } else {
            set_alert('warning', _l('problem_deleting', _l('retention_status_lowercase')));
        }
        redirect(admin_url('retentions/statuses'));
    }

    /* Add new retention note */
    public function add_note($rel_id)
    {
        if (!is_staff_member() || !$this->retentions_model->staff_can_access_retention($rel_id)) {
            ajax_access_denied();
        }

        if ($this->input->post()) {
            $data = $this->input->post();

            // if ($data['contacted_indicator'] == 'yes') {
            //     $contacted_date         = to_sql_date($data['custom_contact_date'], true);
            //     $data['date_contacted'] = $contacted_date;
            // }

            unset($data['contacted_indicator']);
            unset($data['custom_contact_date']);

            // Causing issues with duplicate ID or if my prefixed file for retention.php is used
            $data['description'] = isset($data['retention_note_description']) ? $data['retention_note_description'] : $data['description'];

            if (isset($data['retention_note_description'])) {
                unset($data['retention_note_description']);
            }

            $note_id = $this->misc_model->add_note($data, 'retention', $rel_id);

            if ($note_id) {
                if (isset($contacted_date)) {
                    $this->db->where('id', $rel_id);
                    $this->db->update(db_prefix() . 'retentions', [
                        'lastcontact' => $contacted_date,
                    ]);
                    if ($this->db->affected_rows() > 0) {
                        $this->retentions_model->log_retention_activity($rel_id, 'not_retention_activity_contacted', false, serialize([
                            get_staff_full_name(get_staff_user_id()),
                            _dt($contacted_date),
                        ]));
                    }
                }
            }
        }
        echo json_encode(['retentionView' => $this->_get_retention_data($rel_id), 'id' => $rel_id]);
    }

    public function email_integration_folders()
    {
        if (!is_admin()) {
            ajax_access_denied('retentions Test Email Integration');
        }

        app_check_imap_open_function();

        $imap = new Imap(
            $this->input->post('email'),
            $this->input->post('password', false),
            $this->input->post('imap_server'),
            $this->input->post('encryption')
        );

        try {
            echo json_encode($imap->getSelectableFolders());
        } catch (ConnectionErrorException $e) {
            echo json_encode([
                'alert_type' => 'warning',
                'message'    => $e->getMessage(),
            ]);
        }
    }

    public function test_email_integration()
    {
        if (!is_admin()) {
            access_denied('retentions Test Email Integration');
        }

        app_check_imap_open_function(admin_url('retentions/email_integration'));

        $mail     = $this->retentions_model->get_email_integration();
        $password = $mail->password;

        if (false == $this->encryption->decrypt($password)) {
            set_alert('danger', _l('failed_to_decrypt_password'));
            redirect(admin_url('retentions/email_integration'));
        }

        $imap = new Imap(
            $mail->email,
            $this->encryption->decrypt($password),
            $mail->imap_server,
            $mail->encryption
        );

        try {
            $connection = $imap->testConnection();

            try {
                $connection->getMailbox($mail->folder);
                set_alert('success', _l('retention_email_connection_ok'));
            } catch (MailboxDoesNotExistException $e) {
                set_alert('danger', str_replace(["\n", 'Mailbox'], ['<br />', 'Folder'], addslashes($e->getMessage())));
            }
        } catch (ConnectionErrorException $e) {
            $error = str_replace("\n", '<br />', addslashes($e->getMessage()));
            set_alert('danger', _l('retention_email_connection_not_ok') . '<br /><br /><b>' . $error . '</b>');
        }

        redirect(admin_url('retentions/email_integration'));
    }

    public function email_integration()
    {
        if (!is_admin()) {
            access_denied('retentions Email Intregration');
        }
        if ($this->input->post()) {
            $data             = $this->input->post();
            $data['password'] = $this->input->post('password', false);

            if (isset($data['fakeusernameremembered'])) {
                unset($data['fakeusernameremembered']);
            }
            if (isset($data['fakepasswordremembered'])) {
                unset($data['fakepasswordremembered']);
            }

            $success = $this->retentions_model->update_email_integration($data);
            if ($success) {
                set_alert('success', _l('retentions_email_integration_updated'));
            }
            redirect(admin_url('retentions/email_integration'));
        }
        $data['roles']    = $this->roles_model->get();
        $data['sources']  = $this->retentions_model->get_source();
        $data['statuses'] = $this->retentions_model->get_status();

        $data['members'] = $this->staff_model->get('', [
            'active'       => 1,
            'is_not_staff' => 0,
        ]);

        $data['title'] = _l('retentions_email_integration');
        $data['mail']  = $this->retentions_model->get_email_integration();

        $data['bodyclass'] = 'retentions-email-integration';
        $this->load->view('admin/retentions/email_integration', $data);
    }

    public function change_status_color()
    {
        if ($this->input->post() && is_admin()) {
            $this->retentions_model->change_status_color($this->input->post());
        }
    }

    public function import()
    {
        if (!is_admin() && get_option('allow_non_admin_members_to_import_retentions') != '1') {
            access_denied('retentions Import');
        }

        $dbFields = $this->db->list_fields(db_prefix() . 'retentions');
        array_push($dbFields, 'tags');

        $this->load->library('import/import_retentions', [], 'import');
        $this->import->setDatabaseFields($dbFields)
        ->setCustomFields(get_custom_fields('retentions'));

        if ($this->input->post('download_sample') === 'true') {
            $this->import->downloadSample();
        }

        if ($this->input->post()
            && isset($_FILES['file_csv']['name']) && $_FILES['file_csv']['name'] != '') {
            $this->import->setSimulation($this->input->post('simulate'))
                          ->setTemporaryFileLocation($_FILES['file_csv']['tmp_name'])
                          ->setFilename($_FILES['file_csv']['name'])
                          ->perform();

            $data['total_rows_post'] = $this->import->totalRows();

            if (!$this->import->isSimulation()) {
                set_alert('success', _l('import_total_imported', $this->import->totalImported()));
            }
        }

        $data['statuses'] = $this->retentions_model->get_status();
        $data['sources']  = $this->retentions_model->get_source();
        $data['members']  = $this->staff_model->get('', ['is_not_staff' => 0, 'active' => 1]);

        $data['title'] = _l('import');
        $this->load->view('admin/retentions/import', $data);
    }

    public function validate_unique_field()
    {
        if ($this->input->post()) {

            // First we need to check if the field is the same
            $retention_id = $this->input->post('retention_id');
            $field   = $this->input->post('field');
            $value   = $this->input->post($field);

            if ($retention_id != '') {
                $this->db->select($field);
                $this->db->where('id', $retention_id);
                $row = $this->db->get(db_prefix() . 'retentions')->row();
                if ($row->{$field} == $value) {
                    echo json_encode(true);
                    die();
                }
            }

            echo total_rows(db_prefix() . 'retentions', [ $field => $value ]) > 0 ? 'false' : 'true';
        }
    }

    public function bulk_action()
    {
        if (!is_staff_member()) {
            ajax_access_denied();
        }

        hooks()->do_action('before_do_bulk_action_for_retentions');
        $total_deleted = 0;
        if ($this->input->post()) {
            $ids                   = $this->input->post('ids');
            $status                = $this->input->post('status');
            $source                = $this->input->post('source');
            $assigned              = $this->input->post('assigned');
            $visibility            = $this->input->post('visibility');
            $tags                  = $this->input->post('tags');
            $last_contact          = $this->input->post('last_contact');
            $lost                  = $this->input->post('lost');
            $has_permission_delete = staff_can('delete',  'retentions');
            $has_permission_edit = staff_can('edit',  'retentions');
            if (is_array($ids)) {
                foreach ($ids as $id) {
                    if ($this->input->post('mass_delete')) {
                        if ($has_permission_delete) {
                            if ($this->retentions_model->delete($id)) {
                                $total_deleted++;
                            }
                        }
                    } else {
                        if ($status || $source || $assigned || $last_contact || $visibility) {
                            $update = [];
                            if ($status) {
                                // We will use the same function to update the status
                                $this->retentions_model->update_retention_status([
                                    'status' => $status,
                                    'retentionid' => $id,
                                ]);
                            }
                            if ($source) {
                                $update['source'] = $source;
                            }
                            if ($assigned) {
                                $update['assigned'] = $assigned;
                            }
                            if ($last_contact) {
                                $last_contact          = to_sql_date($last_contact, true);
                                $update['lastcontact'] = $last_contact;
                            }

                            if ($visibility) {
                                if ($visibility == 'public') {
                                    $update['is_public'] = 1;
                                } else {
                                    $update['is_public'] = 0;
                                }
                            }

                            if (count($update) > 0) {
                                $this->db->where('id', $id);
                                $this->db->update(db_prefix() . 'retentions', $update);
                            }
                        }
                        if ($tags) {
                            handle_tags_save($tags, $id, 'retention');
                        }
                        if ($lost == 'true') {
                            $this->retentions_model->mark_as_lost($id);
                        }
                    }
                }
            }
        }

        if ($this->input->post('mass_delete')) {
            set_alert('success', _l('total_retentions_deleted', $total_deleted));
        }
    }

    public function download_files($retention_id)
    {
        if (!is_staff_member() || !$this->retentions_model->staff_can_access_retention($retention_id)) {
            ajax_access_denied();
        }

        $files = $this->retentions_model->get_retention_attachments($retention_id);

        if (count($files) == 0) {
            redirect(previous_url() ?: $_SERVER['HTTP_REFERER']);
        }

        $path = get_upload_path_by_type('retention') . $retention_id;

        $this->load->library('zip');

        foreach ($files as $file) {
            $this->zip->read_file($path . '/' . $file['file_name']);
        }

        $this->zip->download('files.zip');
        $this->zip->clear_data();
    }
}
