<?php

defined('BASEPATH') or exit('No direct script access allowed');

class sales_merge_fields extends App_merge_fields
{
    public function build()
    {
        return [
                [
                    'name'      => 'sale Name',
                    'key'       => '{sale_name}',
                    'available' => [
                        'sales',
                    ],
                    'templates' => [
                        'gdpr-removal-request-sale',
                    ],
                ],
                [
                    'name'      => 'sale Email',
                    'key'       => '{sale_email}',
                    'available' => [
                        'sales',
                    ],
                     'templates' => [
                        'gdpr-removal-request-sale',
                    ],
                ],
                [
                    'name'      => 'sale Position',
                    'key'       => '{sale_position}',
                    'available' => [
                        'sales',
                    ],
                     'templates' => [
                        'gdpr-removal-request-sale',
                    ],
                ],
                [
                    'name'      => 'sale Website',
                    'key'       => '{sale_website}',
                    'available' => [
                        'sales',
                    ],
                     'templates' => [
                        'gdpr-removal-request-sale',
                    ],
                ],
                [
                    'name'      => 'sale Description',
                    'key'       => '{sale_description}',
                    'available' => [
                        'sales',
                    ],
                ],
                [
                    'name'      => 'sale Phone Number',
                    'key'       => '{sale_phonenumber}',
                    'available' => [
                        'sales',
                    ],
                     'templates' => [
                        'gdpr-removal-request-sale',
                    ],
                ],
                [
                    'name'      => 'sale Company',
                    'key'       => '{sale_company}',
                    'available' => [
                        'sales',
                    ],
                     'templates' => [
                        'gdpr-removal-request-sale',
                    ],
                ],
                [
                    'name'      => 'sale Country',
                    'key'       => '{sale_country}',
                    'available' => [
                        'sales',
                    ],
                     'templates' => [
                        'gdpr-removal-request-sale',
                    ],
                ],
                [
                    'name'      => 'sale Zip',
                    'key'       => '{sale_zip}',
                    'available' => [
                        'sales',
                    ],
                ],
                [
                    'name'      => 'sale City',
                    'key'       => '{sale_city}',
                    'available' => [
                        'sales',
                    ],
                ],
                [
                    'name'      => 'sale State',
                    'key'       => '{sale_state}',
                    'available' => [
                        'sales',
                    ],
                ],
                [
                    'name'      => 'sale Address',
                    'key'       => '{sale_address}',
                    'available' => [
                        'sales',
                    ],
                     'templates' => [
                        'gdpr-removal-request-sale',
                    ],
                ],
                [
                    'name'      => 'sale Assigned',
                    'key'       => '{sale_assigned}',
                    'available' => [
                        'sales',
                    ],
                ],
                [
                    'name'      => 'sale Status',
                    'key'       => '{sale_status}',
                    'available' => [
                        'sales',
                    ],
                ],
                [
                    'name'      => 'sale Souce',
                    'key'       => '{sale_source}',
                    'available' => [
                        'sales',
                    ],
                ],
                [
                    'name'      => 'sale Link',
                    'key'       => '{sale_link}',
                    'available' => [
                        'sales',
                    ],
                     'templates' => [
                        'gdpr-removal-request-sale',
                    ],
                ],
                [
                    'name'      => is_gdpr() && get_option('gdpr_enable_sale_public_form') == '1' ? 'sale Public Form URL' : '',
                    'key'       => is_gdpr() && get_option('gdpr_enable_sale_public_form') == '1' ? '{sale_public_form_url}' : '',
                    'available' => [

                    ],
                    'templates' => [
                        'new-web-to-sale-form-submitted',
                    ],
                ],
                [
                    'name'      => is_gdpr() && get_option('gdpr_enable_consent_for_sales') == '1' ? 'sale Consent Link' : '',
                    'key'       => is_gdpr() && get_option('gdpr_enable_consent_for_sales') == '1' ? '{sale_public_consent_url}' : '',
                    'available' => [

                    ],
                    'templates' => [
                        'new-web-to-sale-form-submitted',
                    ],
                ],
            ];
    }

    /**
     * sale merge fields
     * @param  mixed $id sale id
     * @return array
     */
    public function format($id)
    {
        $fields = [];

        $fields['{sale_name}']               = '';
        $fields['{sale_email}']              = '';
        $fields['{sale_position}']           = '';
        $fields['{sale_company}']            = '';
        $fields['{sale_country}']            = '';
        $fields['{sale_zip}']                = '';
        $fields['{sale_city}']               = '';
        $fields['{sale_state}']              = '';
        $fields['{sale_address}']            = '';
        $fields['{sale_assigned}']           = '';
        $fields['{sale_status}']             = '';
        $fields['{sale_source}']             = '';
        $fields['{sale_phonenumber}']        = '';
        $fields['{sale_link}']               = '';
        $fields['{sale_website}']            = '';
        $fields['{sale_description}']        = '';
        $fields['{sale_public_form_url}']    = '';
        $fields['{sale_public_consent_url}'] = '';

        if (is_numeric($id)) {
            $this->ci->db->where('id', $id);
            $sale = $this->ci->db->get(db_prefix().'sales')->row();
        } else {
            $sale = $id;
        }

        if (!$sale) {
            return $fields;
        }

        $fields['{sale_public_form_url}']    = sales_public_url($sale->id);
        $fields['{sale_public_consent_url}'] = sale_consent_url($sale->id);
        $fields['{sale_link}']               = admin_url('sales/index/' . $sale->id);
        $fields['{sale_name}']               = e($sale->name);
        $fields['{sale_email}']              = e($sale->email);
        $fields['{sale_position}']           = e($sale->title);
        $fields['{sale_phonenumber}']        = e($sale->phonenumber);
        $fields['{sale_company}']            = e($sale->company);
        $fields['{sale_zip}']                = e($sale->zip);
        $fields['{sale_city}']               = e($sale->city);
        $fields['{sale_state}']              = e($sale->state);
        $fields['{sale_address}']            = e($sale->address);
        $fields['{sale_website}']            = e($sale->website);
        $fields['{sale_description}']        = e($sale->description);

        if ($sale->assigned != 0) {
            $fields['{sale_assigned}'] = e(get_staff_full_name($sale->assigned));
        }

        if ($sale->country != 0) {
            $country                  = get_country($sale->country);
            $fields['{sale_country}'] = e($country->short_name);
        }

        if ($sale->junk == 1) {
            $fields['{sale_status}'] = _l('sale_junk');
        } elseif ($sale->lost == 1) {
            $fields['{sale_status}'] = _l('sale_lost');
        } else {
            $this->ci->db->select('name');
            $this->ci->db->from(db_prefix().'sales_status');
            $this->ci->db->where('id', $sale->status);
            $status = $this->ci->db->get()->row();
            if ($status) {
                $fields['{sale_status}'] = e($status->name);
            }
        }

        $this->ci->db->select('name');
        $this->ci->db->from(db_prefix().'sales_sources');
        $this->ci->db->where('id', $sale->source);
        $source = $this->ci->db->get()->row();
        if ($source) {
            $fields['{sale_source}'] = e($source->name);
        }

        $custom_fields = get_custom_fields('sales');
        foreach ($custom_fields as $field) {
            $fields['{' . $field['slug'] . '}'] = get_custom_field_value($sale->id, $field['id'], 'sales');
        }

        return hooks()->apply_filters('sale_merge_fields', $fields, ['id' => $sale->id, 'sale' => $sale]);
    }
}
