<?php

use app\services\AbstractKanban;

defined('BASEPATH') or exit('No direct script access allowed');

class Breaks_model extends App_Model
{
    public function __construct()
    {
        parent::__construct();
    }


    public function update_on_break_and_mins_late($user_id, $data)
    {
        $this->db->where('id', $user_id);
        $this->db->update(db_prefix() . 'breaks', $data);

        return $this->db->affected_rows() > 0;
    }

    public function check_on_break($staff_id)
    {
        $this->db->select('*');
        $this->db->from('breaks');
        $this->db->where('staff_id', $staff_id);
        $this->db->where('DATE(start_time)', date('Y-m-d')); // Ensure the break is for the current date
        $this->db->where('on_break', 1); // Ensure the break is active
        $query = $this->db->get();
    
        return $query->row_array(); // Return the active break if found
    }


    public function get_staff_breaks()
    {
        $this->db->select('staff_id, staff_name, break_type, start_time, duration');
        $this->db->from('tblbreaks'); // Replace with your actual table name
        $this->db->where('end_time IS NULL'); // Assuming `end_time` is NULL for active breaks
        $this->db->join('tblstaff', 'tblbreaks.staff_id = tblstaff.id'); // Join with staff table if necessary
        $this->db->order_by('start_time', 'DESC'); // Optional: Order by start time

        $query = $this->db->get();
        return $query->result_array();
    }

    

    public function create_staff_break($data)
    {
        // Prepare data for insertion (ensure valid fields are inserted)
        $this->db->insert(db_prefix() . 'breaks', [
            'staff_id' => $data['staff_id'],           // Staff ID
            'break_type' => $data['break_type'],       // Break type
            'break_minutes' => $data['break_minutes'], // Break duration in minutes
            'start_time' => $data['start_time'],       // Break start time
            'end_time' => $data['end_time'],           // Calculated end time
            'created_at' => date('Y-m-d H:i:s'),       // Optional: Add timestamp for the break creation
            'on_break' => 1,
        ]);

        // Return the inserted ID
        // return $this->db->insert_id();
        return true;
    }


    public function update_break($data, $id)
    {
        // Prepare data for updating the record
        $this->db->where('id', $id);
        $this->db->update(db_prefix() . 'break', [
            'break_type' => $data['break_type'],
            'break_minutes' => $data['break_minutes'],
        ]);

        // Return true if the update was successful
        // return $this->db->affected_rows() > 0;
        return true;
    }
    
    public function delete($id)
    {
        // Ensure the ID is numeric
        if (!is_numeric($id)) {
            return false;
        }

        // Hook before delete to allow extensions to the function
        hooks()->do_action('before_delete_break', [
            'id' => $id,
        ]);

        // Get break details for logging purposes
        $break_details = $this->db->where('id', $id)->get(db_prefix() . 'break')->row();
        
        if (!$break_details) {
            return false; // If break doesn't exist, return false
        }

        $break_type = $break_details->break_type;
        $break_minutes = $break_details->break_minutes;

        // Check if there's any associated data related to this break (e.g., break history or logs)
        // Example: If you have a table that tracks when a break was created or logs
       
        // Delete the actual break
        $this->db->where('id', $id);
        $this->db->delete(db_prefix() . 'break');

        // Log the activity
        log_activity('Break Deleted [Type: ' . $break_type . ', Duration: ' . $break_minutes . ' minutes]');

        // Hook after break deletion for any additional actions
        hooks()->do_action('after_delete_break', [
            'id' => $id,
            'break_type' => $break_type,
            'break_minutes' => $break_minutes,
        ]);

        return true;
    }


    /**
     * Get all available breaks from the database
     * @return array
     */
    
    public function get_all_breaks()
    {
        // Query the database to get all the breaks
        $this->db->select('id, break_type, break_minutes');
        $query = $this->db->get(db_prefix() . 'break');
        
        // Return the result as an array
        return $query->result_array();
    }


    public function add_break() {

    }


    // public function update_staff_break_with_late_time($staff_id, $break_type, $mins_late) {
    //     $this->db->where('staff_id', $staff_id);
    //     $this->db->where('break_type', $break_type);
    //     $this->db->order_by('id', 'DESC'); // Ensure you update the latest break entry
    //     $this->db->limit(1); // Limit to the most recent entry

    //     return $this->db->update(db_prefix() . 'breaks', [
    //         'mins_late' => $mins_late, // Update the min_late column with the late time
    //         'on_break' => 0
    //     ]);
    // }
    

    public function update_staff_break_with_late_time($staff_id, $break_type, $mins_late) 
    {
        $this->db->trans_start(); // Start a transaction
    
        $this->db->where('staff_id', $staff_id);
        $this->db->where('break_type', $break_type);
        $this->db->order_by('id', 'DESC'); // Ensure you update the latest break entry
        $this->db->limit(1); // Limit to the most recent entry
    
        $data = [
            'mins_late' => $mins_late, // Update the min_late column with the late time
            'on_break' => 0 // Assume break ends after this call
        ];
    
        $this->db->update(db_prefix() . 'breaks', $data);
    
        $this->db->trans_complete(); // Complete the transaction
    
        // Return the result of the transaction
        if ($this->db->trans_status() === FALSE) {
            return false;
        } else {
            return true;
        }
    }


    // Function to get the end_time for a specific staff member and break
    // Function to get the end_time for a specific staff member and break
public function get_end_time_by_staff($staff_id) {
    // Select the end_time column from the breaks table
    $this->db->select('end_time');
    $this->db->from(db_prefix() . 'breaks'); // Ensure the table name is correct
    $this->db->where('staff_id', $staff_id); // Ensure staff_id is passed correctly
    $this->db->order_by('id', 'DESC'); // Get the most recent break entry
    $this->db->limit(1); // Limit to the most recent entry

    // Execute the query
    $query = $this->db->get();
    
    // Debugging: log the query and the result for troubleshooting
    log_message('debug', 'SQL Query: ' . $this->db->last_query());

    if ($query->num_rows() > 0) {
        $result = $query->row();
        log_message('debug', 'Query Result: ' . print_r($result, true)); // Log the result
        return $result->end_time; // Return the end_time field
    } else {
        log_message('error', 'No end_time found for staff_id: ' . $staff_id); // Log an error
        return false; // No break found
    }
}



    
    
}
