<?php

use app\services\AbstractKanban;

defined('BASEPATH') or exit('No direct script access allowed');

class Targets_model extends App_Model
{
    public function __construct()
    {
        parent::__construct();
    }


    // Insert or update target for a staff member
    public function save_target($user_id, $target, $target_month) {
        // Fetch the staff's firstname from the staff table
        $this->db->select('firstname');
        $this->db->where('staffid', $user_id);
        $staff_query = $this->db->get(db_prefix() . 'staff');
        
        // Check if staff exists and retrieve firstname
        if ($staff_query->num_rows() > 0) {
            $staff = $staff_query->row();
            $staff_firstname = $staff->firstname;
        } else {
            // If the staff doesn't exist, handle the error or return false
            return false;
        }
    
        // Check if the target for the given month already exists
        $this->db->where('user_id', $user_id);
        $this->db->where('target_month', $target_month);
        $query = $this->db->get(db_prefix() . 'targets');
    
        // If target data exists, update it
        if ($query->num_rows() > 0) {
            $data = [
                'target' => $target,
                'updated_at' => date('Y-m-d H:i:s'),
                'full_name' => $staff_firstname, // Save the staff's firstname
            ];
            
            // Use the set method for updating the existing target record
            $this->db->set($data); // Automatically escapes and prepares the data
            $this->db->where('user_id', $user_id);
            $this->db->where('target_month', $target_month);
            return $this->db->update(db_prefix() . 'targets');
        } else {
            // Otherwise, insert new target record
            $data = [
                'user_id'       => $user_id,
                'target'        => $target,
                'target_month'  => $target_month,
                'created_at'    => date('Y-m-d H:i:s'),
                'updated_at'    => date('Y-m-d H:i:s'),
                'full_name'=> $staff_firstname, // Save the staff's firstname
            ];
            
            // Insert new record
            return $this->db->insert(db_prefix() . 'targets', $data);
        }
    }
    
    


    

    public function create_staff_break($data)
    {
        // Prepare data for insertion (ensure valid fields are inserted)
        $this->db->insert(db_prefix() . 'breaks', [
            'staff_id' => $data['staff_id'],           // Staff ID
            'break_type' => $data['break_type'],       // Break type
            'break_minutes' => $data['break_minutes'], // Break duration in minutes
            'start_time' => $data['start_time'],       // Break start time
            'end_time' => $data['end_time'],           // Calculated end time
            'created_at' => date('Y-m-d H:i:s'),       // Optional: Add timestamp for the break creation
            'on_break' => 1,
        ]);

        // Return the inserted ID
        // return $this->db->insert_id();
        return true;
    }


    public function update_break($data, $id)
    {
        // Prepare data for updating the record
        $this->db->where('id', $id);
        $this->db->update(db_prefix() . 'break', [
            'break_type' => $data['break_type'],
            'break_minutes' => $data['break_minutes'],
        ]);

        // Return true if the update was successful
        // return $this->db->affected_rows() > 0;
        return true;
    }
    
    public function delete($id)
    {
        // Ensure the ID is numeric
        if (!is_numeric($id)) {
            return false;
        }

        // Hook before delete to allow extensions to the function
        hooks()->do_action('before_delete_break', [
            'id' => $id,
        ]);

        // Get break details for logging purposes
        $break_details = $this->db->where('id', $id)->get(db_prefix() . 'break')->row();
        
        if (!$break_details) {
            return false; // If break doesn't exist, return false
        }

        $break_type = $break_details->break_type;
        $break_minutes = $break_details->break_minutes;

        // Check if there's any associated data related to this break (e.g., break history or logs)
        // Example: If you have a table that tracks when a break was created or logs
       
        // Delete the actual break
        $this->db->where('id', $id);
        $this->db->delete(db_prefix() . 'break');

        // Log the activity
        log_activity('Break Deleted [Type: ' . $break_type . ', Duration: ' . $break_minutes . ' minutes]');

        // Hook after break deletion for any additional actions
        hooks()->do_action('after_delete_break', [
            'id' => $id,
            'break_type' => $break_type,
            'break_minutes' => $break_minutes,
        ]);

        return true;
    }


    /**
     * Get all available breaks from the database
     * @return array
     */
    
    public function get_all_breaks()
    {
        // Query the database to get all the breaks
        $this->db->select('id, break_type, break_minutes');
        $query = $this->db->get(db_prefix() . 'break');
        
        // Return the result as an array
        return $query->result_array();
    }


    public function add_break() {

    }


    // public function update_staff_break_with_late_time($staff_id, $break_type, $mins_late) {
    //     $this->db->where('staff_id', $staff_id);
    //     $this->db->where('break_type', $break_type);
    //     $this->db->order_by('id', 'DESC'); // Ensure you update the latest break entry
    //     $this->db->limit(1); // Limit to the most recent entry

    //     return $this->db->update(db_prefix() . 'breaks', [
    //         'mins_late' => $mins_late, // Update the min_late column with the late time
    //         'on_break' => 0
    //     ]);
    // }
    

    public function update_staff_break_with_late_time($staff_id, $break_type, $mins_late) 
    {
        $this->db->trans_start(); // Start a transaction
    
        $this->db->where('staff_id', $staff_id);
        $this->db->where('break_type', $break_type);
        $this->db->order_by('id', 'DESC'); // Ensure you update the latest break entry
        $this->db->limit(1); // Limit to the most recent entry
    
        $data = [
            'mins_late' => $mins_late, // Update the min_late column with the late time
            'on_break' => 0 // Assume break ends after this call
        ];
    
        $this->db->update(db_prefix() . 'breaks', $data);
    
        $this->db->trans_complete(); // Complete the transaction
    
        // Return the result of the transaction
        if ($this->db->trans_status() === FALSE) {
            return false;
        } else {
            return true;
        }
    }


    // Function to get the end_time for a specific staff member and break
    // Function to get the end_time for a specific staff member and break
public function get_end_time_by_staff($staff_id) {
    // Select the end_time column from the breaks table
    $this->db->select('end_time');
    $this->db->from(db_prefix() . 'breaks'); // Ensure the table name is correct
    $this->db->where('staff_id', $staff_id); // Ensure staff_id is passed correctly
    $this->db->order_by('id', 'DESC'); // Get the most recent break entry
    $this->db->limit(1); // Limit to the most recent entry

    // Execute the query
    $query = $this->db->get();
    
    // Debugging: log the query and the result for troubleshooting
    log_message('debug', 'SQL Query: ' . $this->db->last_query());

    if ($query->num_rows() > 0) {
        $result = $query->row();
        log_message('debug', 'Query Result: ' . print_r($result, true)); // Log the result
        return $result->end_time; // Return the end_time field
    } else {
        log_message('error', 'No end_time found for staff_id: ' . $staff_id); // Log an error
        return false; // No break found
    }
}



    
    
}
