<?php

namespace app\services\retentions;

use app\services\AbstractKanban;

class retentionsKanban extends AbstractKanban
{
    protected function table(): string
    {
        return 'retentions';
    }

    public function defaultSortDirection()
    {
        return get_option('default_retentions_kanban_sort_type');
    }

    public function defaultSortColumn()
    {
        return get_option('default_retentions_kanban_sort');
    }

    public function limit()
    {
        return get_option('retentions_kanban_limit');
    }

    protected function applySearchQuery($q): self
    {
        if (!startsWith($q, '#')) {
            $q = $this->ci->db->escape_like_str($this->q);
            $this->ci->db->where('(' . db_prefix() . 'retentions.name LIKE "%' . $q . '%" ESCAPE \'!\' OR ' . db_prefix() . 'retentions_sources.name LIKE "%' . $q . '%" ESCAPE \'!\' OR ' . db_prefix() . 'retentions.email LIKE "%' . $q . '%" ESCAPE \'!\' OR ' . db_prefix() . 'retentions.phonenumber LIKE "%' . $q . '%" ESCAPE \'!\' OR ' . db_prefix() . 'retentions.company LIKE "%' . $q . '%" ESCAPE \'!\' OR CONCAT(' . db_prefix() . 'staff.firstname, \' \', ' . db_prefix() . 'staff.lastname) LIKE "%' . $q . '%" ESCAPE \'!\')');
        } else {
            $this->ci->db->where(db_prefix() . 'retentions.id IN
                (SELECT rel_id FROM ' . db_prefix() . 'taggables WHERE tag_id IN
                (SELECT id FROM ' . db_prefix() . 'tags WHERE name="' . $this->ci->db->escape_str(strafter($q, '#')) . '")
                AND ' . db_prefix() . 'taggables.rel_type=\'retention\' GROUP BY rel_id HAVING COUNT(tag_id) = 1)
                ');
        }

        return $this;
    }

    protected function initiateQuery(): self
    {
        $this->ci->db->select(db_prefix() . 'retentions.title, ' . db_prefix() . 'retentions.website, ' . db_prefix() . 'retentions.retention_value, ' . db_prefix() . 'retentions.address, ' . db_prefix() . 'retentions.city, ' . db_prefix() . 'retentions.state, ' . db_prefix() . 'retentions.country, ' . db_prefix() . 'retentions.zip, ' . db_prefix() . 'retentions.name as retention_name,' . db_prefix() . 'retentions_sources.name as source_name,' . db_prefix() . 'retentions.id as id,' . db_prefix() . 'retentions.assigned,' . db_prefix() . 'retentions.email,' . db_prefix() . 'retentions.phonenumber,' . db_prefix() . 'retentions.company,' . db_prefix() . 'retentions.dateadded,' . db_prefix() . 'retentions.status,' . db_prefix() . 'retentions.lastcontact,(SELECT COUNT(*) FROM ' . db_prefix() . 'clients WHERE retentionid=' . db_prefix() . 'retentions.id) as is_retention_client, (SELECT COUNT(id) FROM ' . db_prefix() . 'files WHERE rel_id=' . db_prefix() . 'retentions.id AND rel_type="retention") as total_files, (SELECT COUNT(id) FROM ' . db_prefix() . 'notes WHERE rel_id=' . db_prefix() . 'retentions.id AND rel_type="retention") as total_notes,(SELECT GROUP_CONCAT(name SEPARATOR ",") FROM ' . db_prefix() . 'taggables JOIN ' . db_prefix() . 'tags ON ' . db_prefix() . 'taggables.tag_id = ' . db_prefix() . 'tags.id WHERE rel_id = ' . db_prefix() . 'retentions.id and rel_type="retention" ORDER by tag_order ASC) as tags');
        $this->ci->db->from('retentions');
        $this->ci->db->join(db_prefix() . 'retentions_sources', db_prefix() . 'retentions_sources.id=' . db_prefix() . 'retentions.source');
        $this->ci->db->join(db_prefix() . 'staff', db_prefix() . 'staff.staffid=' . db_prefix() . 'retentions.assigned', 'left');
        $this->ci->db->where('status', $this->status);

        if (staff_cant('view', 'retentions')) {
            $this->ci->db->where('(assigned = ' . get_staff_user_id() . ' OR addedfrom=' . get_staff_user_id() . ' OR is_public=1)');
        }

        return $this;
    }
}
