<?php

namespace app\services\sales;

use app\services\AbstractKanban;

class salesKanban extends AbstractKanban
{
    protected function table(): string
    {
        return 'sales';
    }

    public function defaultSortDirection()
    {
        return get_option('default_sales_kanban_sort_type');
    }

    public function defaultSortColumn()
    {
        return get_option('default_sales_kanban_sort');
    }

    public function limit()
    {
        return get_option('sales_kanban_limit');
    }

    protected function applySearchQuery($q): self
    {
        if (!startsWith($q, '#')) {
            $q = $this->ci->db->escape_like_str($this->q);
            $this->ci->db->where('(' . db_prefix() . 'sales.name LIKE "%' . $q . '%" ESCAPE \'!\' OR ' . db_prefix() . 'sales_sources.name LIKE "%' . $q . '%" ESCAPE \'!\' OR ' . db_prefix() . 'sales.email LIKE "%' . $q . '%" ESCAPE \'!\' OR ' . db_prefix() . 'sales.phonenumber LIKE "%' . $q . '%" ESCAPE \'!\' OR ' . db_prefix() . 'sales.company LIKE "%' . $q . '%" ESCAPE \'!\' OR CONCAT(' . db_prefix() . 'staff.firstname, \' \', ' . db_prefix() . 'staff.lastname) LIKE "%' . $q . '%" ESCAPE \'!\')');
        } else {
            $this->ci->db->where(db_prefix() . 'sales.id IN
                (SELECT rel_id FROM ' . db_prefix() . 'taggables WHERE tag_id IN
                (SELECT id FROM ' . db_prefix() . 'tags WHERE name="' . $this->ci->db->escape_str(strafter($q, '#')) . '")
                AND ' . db_prefix() . 'taggables.rel_type=\'sale\' GROUP BY rel_id HAVING COUNT(tag_id) = 1)
                ');
        }

        return $this;
    }

    protected function initiateQuery(): self
    {
        $this->ci->db->select(db_prefix() . 'sales.title, ' . db_prefix() . 'sales.website, ' . db_prefix() . 'sales.sale_value, ' . db_prefix() . 'sales.address, ' . db_prefix() . 'sales.city, ' . db_prefix() . 'sales.state, ' . db_prefix() . 'sales.country, ' . db_prefix() . 'sales.zip, ' . db_prefix() . 'sales.name as sale_name,' . db_prefix() . 'sales_sources.name as source_name,' . db_prefix() . 'sales.id as id,' . db_prefix() . 'sales.assigned,' . db_prefix() . 'sales.email,' . db_prefix() . 'sales.phonenumber,' . db_prefix() . 'sales.company,' . db_prefix() . 'sales.dateadded,' . db_prefix() . 'sales.status,' . db_prefix() . 'sales.lastcontact,(SELECT COUNT(*) FROM ' . db_prefix() . 'clients WHERE saleid=' . db_prefix() . 'sales.id) as is_sale_client, (SELECT COUNT(id) FROM ' . db_prefix() . 'files WHERE rel_id=' . db_prefix() . 'sales.id AND rel_type="sale") as total_files, (SELECT COUNT(id) FROM ' . db_prefix() . 'notes WHERE rel_id=' . db_prefix() . 'sales.id AND rel_type="sale") as total_notes,(SELECT GROUP_CONCAT(name SEPARATOR ",") FROM ' . db_prefix() . 'taggables JOIN ' . db_prefix() . 'tags ON ' . db_prefix() . 'taggables.tag_id = ' . db_prefix() . 'tags.id WHERE rel_id = ' . db_prefix() . 'sales.id and rel_type="sale" ORDER by tag_order ASC) as tags');
        $this->ci->db->from('sales');
        $this->ci->db->join(db_prefix() . 'sales_sources', db_prefix() . 'sales_sources.id=' . db_prefix() . 'sales.source');
        $this->ci->db->join(db_prefix() . 'staff', db_prefix() . 'staff.staffid=' . db_prefix() . 'sales.assigned', 'left');
        $this->ci->db->where('status', $this->status);

        if (staff_cant('view', 'sales')) {
            $this->ci->db->where('(assigned = ' . get_staff_user_id() . ' OR addedfrom=' . get_staff_user_id() . ' OR is_public=1)');
        }

        return $this;
    }
}
