<?php

defined('BASEPATH') or exit('No direct script access allowed');

$has_permission_delete = staff_can('delete', 'break');

$aColumns = [
    'break_type',   // Column for break type (e.g., 'Lunch Break', 'Coffee Break')
    'break_minutes',  // Column for break duration in minutes
    'created_at',   // Date when the break was taken
];

$sIndexColumn = 'id';
$sTable       = db_prefix() . 'break';

// No join needed for this since only breaks table data is required
$join = [];

$where = hooks()->apply_filters('break_table_sql_where', []);

$result = data_tables_init($aColumns, $sIndexColumn, $sTable, $join, $where, [
    'id',
    'user_id',
]);

$output  = $result['output'];
$rResult = $result['rResult'];

foreach ($rResult as $aRow) {
    $row = [];
    for ($i = 0; $i < count($aColumns); $i++) {
        $_data = $aRow[$aColumns[$i]];
        
        if ($aColumns[$i] == 'created_at') {
            $_data = '<span class="text-has-action is-date" data-toggle="tooltip" data-title="' . e(_dt($_data)) . '">' . time_ago($_data) . '</span>';
        }

        // Handle row options (view, delete) for each break
        if ($aColumns[$i] == 'break_type') {
            $_data = '<a href="' . admin_url('break/view/' . $aRow['id']) . '">' . e($aRow['break_type']) . '</a>';
            $_data .= '<div class="row-options">';
            $_data .= '<a href="' . admin_url('break/view/' . $aRow['id']) . '">' . _l('view') . '</a>';
            
            if ($has_permission_delete) {
                $_data .= ' | <a href="#" onclick="delete_break(' . $aRow['id'] . '); return false;" class="text-danger">' . _l('delete') . '</a>';
            }
            
            $_data .= '</div>';
        }

        $row[] = $_data;
    }

    $row['DT_RowClass'] = 'has-row-options';

    $row = hooks()->apply_filters('break_table_row', $row, $aRow);

    $output['aaData'][] = $row;
}
