<?php

defined('BASEPATH') or exit('No direct script access allowed');

$has_permission_delete = staff_can('delete', 'break');

$aColumns = [
    'tblbreaks.id as id',           // Column for the break ID
    'CONCAT(tblstaff.firstname, " ", tblstaff.lastname) as staff_name', // Column for staff name
    'tblbreaks.break_type as break_type',    // Column for the type of break
    'tblbreaks.break_minutes as break_minutes', // Column for break duration in minutes
    'tblbreaks.start_time as start_time',    // Column for the start time of the break
    'tblbreaks.end_time as end_time',        // Column for the end time of the break
    'tblbreaks.on_break as on_break',        // Column for whether the break is ongoing
    'tblbreaks.created_at as created_at',    // Column for the creation date of the break
    'tblbreaks.mins_late as mins_late',      // Column for late minutes
];

$sIndexColumn = 'id';
$sTable       = db_prefix() . 'breaks';

// Add a join to fetch staff firstname and lastname
$join = [
    'LEFT JOIN ' . db_prefix() . 'staff as tblstaff ON tblstaff.staffid = tblbreaks.staff_id',
];

// Determine if the user is an admin
$is_admin = is_admin();

// Add a WHERE condition to filter by the current staff ID if not an admin
$where = [];
if (!$is_admin) {
    $where[] = 'AND tblbreaks.staff_id = ' . get_staff_user_id();
}

// Allow hooks to modify the WHERE condition
$where = hooks()->apply_filters('break_table_sql_where', $where);

$result = data_tables_init($aColumns, $sIndexColumn, $sTable, $join, $where);

$output  = $result['output'];
$rResult = $result['rResult'];

foreach ($rResult as $aRow) {
    $row = [];
    foreach ($aColumns as $column) {
        $column_alias = explode(' as ', $column); // Handle alias
        $column_key = count($column_alias) > 1 ? $column_alias[1] : $column; // Get alias if exists, else original column

        $_data = $aRow[$column_key];

        if ($column_key == 'created_at') {
            $_data = '<span class="text-has-action is-date" data-toggle="tooltip" data-title="' . e(_dt($_data)) . '">' . time_ago($_data) . '</span>';
        }

        if ($column_key == 'on_break') {
            if ($is_admin) {
                // Render dropdown for admin users
                $_data = '<div class="dropdown">
                              <button class="btn btn-sm dropdown-toggle" type="button" id="dropdownMenuButton' . $aRow['id'] . '" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">
                                  ' . ($aRow['on_break'] ? _l('Yes') : _l('No')) . '
                              </button>
                              <div class="dropdown-menu" aria-labelledby="dropdownMenuButton' . $aRow['id'] . '">
                                  <a class="dropdown-item" href="#" onclick="updateOnBreakStatus(' . $aRow['id'] . ', 1); return false;">' . _l('Yes') . '</a>
                                  <a class="dropdown-item" href="#" onclick="updateOnBreakStatus(' . $aRow['id'] . ', 0); return false;">' . _l('No') . '</a>
                              </div>
                          </div>';
            } else {
                // Display static text for non-admin users
                $_data = $aRow['on_break'] ? _l('Yes') : _l('No');
            }
        }

        if ($column_key == 'break_type') {
            $_data = '<a href="' . admin_url('break/view/' . $aRow['id']) . '">' . e($aRow['break_type']) . '</a>';
            $_data .= '<div class="row-options">';
            $_data .= '<a href="' . admin_url('break/view/' . $aRow['id']) . '">' . _l('view') . '</a>';
            
            if ($has_permission_delete) {
                $_data .= ' | <a href="#" onclick="delete_break(' . $aRow['id'] . '); return false;" class="text-danger">' . _l('delete') . '</a>';
            }
            
            $_data .= '</div>';
        }

        if ($column_key == 'start_time' || $column_key == 'end_time') {
            $_data = $_data ? _dt($_data) : '-';
        }

        if ($column_key == 'staff_name') {
            $_data = e($aRow['staff_name']);
        }

        $row[] = $_data;
    }

    $row['DT_RowClass'] = 'has-row-options';

    $row = hooks()->apply_filters('break_table_row', $row, $aRow);

    $output['aaData'][] = $row;
}

?>
