<?php

defined('BASEPATH') or exit('No direct script access allowed');

$this->ci->load->model('gdpr_model');
$this->ci->load->model('sales_model');
$this->ci->load->model('staff_model');
$statuses = $this->ci->sales_model->get_status();
// $get_where_report_period = $this->ci->sales_model->get_where_report_period();

if (is_gdpr() && get_option('gdpr_enable_consent_for_sales') == '1') {
    $consent_purposes = $this->ci->gdpr_model->get_consent_purposes();
}

$rules = [
    App_table_filter::new('name', 'TextRule')->label(_l('sales_dt_name')),
    App_table_filter::new('phonenumber', 'TextRule')->label(_l('sales_dt_phonenumber')),
    App_table_filter::new('country', 'SelectRule')->label(_l('sale_country'))->options(function ($ci) {
        return collect(get_all_countries())->map(fn ($country) => [
            'value' => $country['country_id'],
            'label' => $country['short_name'],
        ]);
    }),
    App_table_filter::new('city', 'TextRule')->label(_l('sale_city')),
    App_table_filter::new('state', 'TextRule')->label(_l('sale_state')),
    App_table_filter::new('zip', 'TextRule')->label(_l('sale_zip')),
    App_table_filter::new('is_public', 'BooleanRule')->label(_l('sale_public')),
    App_table_filter::new('lost', 'BooleanRule')->label(_l('sale_lost')),
    App_table_filter::new('junk', 'BooleanRule')->label(_l('sale_junk')),
    App_table_filter::new('lastcontact', 'DateRule')->label(_l('sales_dt_last_contact')),
    App_table_filter::new('dateadded', 'DateRule')->label(_l('date_created')),
    App_table_filter::new('dateassigned', 'DateRule')->label(_l('customer_admin_date_assigned')),
    App_table_filter::new('last_status_change', 'DateRule')->label(_l('lead_last_status_change')),
    App_table_filter::new('sale_value', 'NumberRule')->label(_l('sale_add_edit_sale_value')),
    App_table_filter::new('status', 'MultiSelectRule')->label(_l('sale_status'))->options(function () use ($statuses) {
        return collect($statuses)->map(fn ($status) => [
            'value' => $status['id'],
            'label' => $status['name'],
            'subtext' => $status['isdefault'] == 1 ? _l('sales_converted_to_client') : null,
        ]);
    }),
    App_table_filter::new('source', 'MultiSelectRule')->label(_l('sale_source'))->options(function ($ci) {
        return collect($ci->sales_model->get_source())->map(fn ($source) => [
            'value' => $source['id'],
            'label' => $source['name'],
        ]);
    }),
];

$rules[] = App_table_filter::new('assigned', 'SelectRule')->label(_l('sales_dt_assigned'))
    ->withEmptyOperators()
    ->emptyOperatorValue(0)
    ->isVisible(fn () => staff_can('view', 'sales'))
    ->options(function ($ci) {
        $staff = $ci->staff_model->get('', ['active' => 1]);

        return collect($staff)->map(function ($staff) {
            return [
                'value' => $staff['staffid'],
                'label' => $staff['firstname'] . ' ' . $staff['lastname']
            ];
        })->all();
    });


if (isset($consent_purposes)) {
    $rules[] = App_table_filter::new('gdpr_content', 'SelectRule')
        ->label(_l('gdpr_consent'))
        ->options(function () use ($consent_purposes) {
            return collect($consent_purposes)->map(fn ($purpose) => [
                'value' => $purpose['id'],
                'label' => $purpose['name']
            ]);
        })->raw(function ($value, $operator, $sql_operator) {
            return db_prefix() . 'sales.id ' . $sql_operator . ' (SELECT sale_id FROM ' . db_prefix() . 'consents WHERE purpose_id=' . $value . ' and action="opt-in" AND date IN (SELECT MAX(date) FROM ' . db_prefix() . 'consents WHERE purpose_id=' . $value . ' AND sale_id=' . db_prefix() . 'sales.id))';
        });
}

return App_table::find('sales')
    ->outputUsing(function ($params) use ($statuses) {
        extract($params);

        $lockAfterConvert      = get_option('sale_lock_after_convert_to_customer');
        $has_permission_delete = staff_can('delete',  'sales');
        $custom_fields         = get_table_custom_fields('sales');
        $consentsales          = get_option('gdpr_enable_consent_for_sales');

        $aColumns = [
            '1',
            db_prefix() . 'sales.id as id',
            db_prefix() . 'sales.name as name',
        ];
        if (is_gdpr() && $consentsales == '1') {
            $aColumns[] = '1';
        }
        $aColumns = array_merge($aColumns, [
            'company',
            db_prefix() . 'sales.email as email',
            db_prefix() . 'sales.phonenumber as phonenumber',
            'sale_value',
            '(SELECT GROUP_CONCAT(name SEPARATOR ",") FROM ' . db_prefix() . 'taggables JOIN ' . db_prefix() . 'tags ON ' . db_prefix() . 'taggables.tag_id = ' . db_prefix() . 'tags.id WHERE rel_id = ' . db_prefix() . 'sales.id and rel_type="sale" ORDER by tag_order ASC LIMIT 1) as tags',
            'firstname as assigned_firstname',
            db_prefix() . 'sales_status.name as status_name',
            db_prefix() . 'sales_sources.name as source_name',
            'lastcontact',
            'dateadded',
        ]);

        $sIndexColumn = 'id';
        $sTable       = db_prefix() . 'sales';

        $join = [
            'LEFT JOIN ' . db_prefix() . 'staff ON ' . db_prefix() . 'staff.staffid = ' . db_prefix() . 'sales.assigned',
            'LEFT JOIN ' . db_prefix() . 'sales_status ON ' . db_prefix() . 'sales_status.id = ' . db_prefix() . 'sales.status',
            'JOIN ' . db_prefix() . 'sales_sources ON ' . db_prefix() . 'sales_sources.id = ' . db_prefix() . 'sales.source',
        ];

        foreach ($custom_fields as $key => $field) {
            $selectAs = (is_cf_date($field) ? 'date_picker_cvalue_' . $key : 'cvalue_' . $key);
            array_push($customFieldsColumns, $selectAs);
            array_push($aColumns, 'ctable_' . $key . '.value as ' . $selectAs);
            array_push($join, 'LEFT JOIN ' . db_prefix() . 'customfieldsvalues as ctable_' . $key . ' ON ' . db_prefix() . 'sales.id = ctable_' . $key . '.relid AND ctable_' . $key . '.fieldto="' . $field['fieldto'] . '" AND ctable_' . $key . '.fieldid=' . $field['id']);
        }

        $where  = [];

        if ($filtersWhere = $this->getWhereFromRules()) {
            $where[] = $filtersWhere;
        }
        
        if ($this->ci->input->post('custom_view')) {
            $filter = $this->ci->input->post('custom_view');
            if ($filter == 'lost') {
                array_push($where, 'AND lost = 1');
            } elseif ($filter == 'junk') {
                array_push($where, 'AND junk = 1');
            } elseif ($filter == 'not_assigned') {
                array_push($where, 'AND assigned = 0');
            } elseif ($filter == 'contacted_today') {
                array_push($where, 'AND lastcontact LIKE "'.date('Y-m-d').'%"');
            } elseif ($filter == 'created_today') {
                array_push($where, 'AND dateadded LIKE "'.date('Y-m-d').'%"');
            } elseif ($filter == 'public') {
                array_push($where, 'AND is_public = 1');
            }
        }
        $filter = $this->ci->input->post('custom_view');

        if (!$filter || ($filter && $filter != 'lost' && $filter != 'junk')) {
            array_push($where, 'AND lost = 0 AND junk = 0');
        }

        if (has_permission('sales','','view') && $this->ci->input->post('assigned')) {
            array_push($where, 'AND assigned =' . $this->ci->input->post('assigned'));
        }
                
                if ($this->ci->input->post('status')
            && count($this->ci->input->post('status')) > 0
            && ($filter != 'lost' && $filter != 'junk')) {
            array_push($where, 'AND status IN ('.implode(',',$this->ci->input->post('status')).')');
        }

                if ($this->ci->input->post('source')
            && count($this->ci->input->post('source')) > 0) {
            array_push($where, 'AND source IN ('.implode(',',$this->ci->input->post('source')).')');
        }



        if ($this->ci->input->post('countries')) {
            array_push($where, 'AND country IN ('.implode(',',$this->ci->input->post('countries')).')');
        }
        if ($this->ci->input->post('name')) {
            array_push($where, 'AND ' . db_prefix() . 'sales.name LIKE "%' . $this->ci->input->post('name') . '%"');
        }

        if ($this->ci->input->post('phone')) {
            array_push($where, 'AND ' . db_prefix() . 'sales.phonenumber LIKE "%' . $this->ci->input->post('phone') . '%"');
        }

        if ($this->ci->input->post('id')) {
            array_push($where, 'AND ' . db_prefix() . 'sales.id LIKE "%' . $this->ci->input->post('id') . '%"');
        }

        if ($this->ci->input->post('report_months')) {
            $custom_date_select = $this->ci->sales_model->get_where_report_period('DATE('.$this->ci->input->post('date_by').')',$this->ci->input->post('report_months'));
        
            array_push($where, $custom_date_select);
        }

        if ($this->ci->input->post('tags')) {
            $tags = $this->ci->input->post('tags');
            $tags_imploded = implode(',', $tags);

            // Use a subquery or a separate join to handle tags filtering
            $tagsFilter = 'EXISTS (SELECT 1 FROM ' . db_prefix() . 'taggables as t JOIN ' . db_prefix() . 'tags as tag ON t.tag_id = tag.id WHERE t.rel_id = ' . db_prefix() . 'sales.id AND t.rel_type = "sale" AND t.tag_id IN (' . $tags_imploded . '))';
            array_push($where, 'AND ' . $tagsFilter);
        }



        if (staff_cant('view', 'sales')) {
            array_push($where, 'AND (assigned =' . get_staff_user_id() . ' OR addedfrom = ' . get_staff_user_id() . ' OR is_public = 1)');
        }

        $aColumns = hooks()->apply_filters('sales_table_sql_columns', $aColumns);

        // Fix for big queries. Some hosting have max_join_limit
        if (count($custom_fields) > 4) {
            @$this->ci->db->query('SET SQL_BIG_SELECTS=1');
        }

        $additionalColumns = hooks()->apply_filters('sales_table_additional_columns_sql', [
            'junk',
            'lost',
            'color',
            'status',
            'assigned',
            'lastname as assigned_lastname',
            db_prefix() . 'sales.addedfrom as addedfrom',
            '(SELECT count(saleid) FROM ' . db_prefix() . 'clients WHERE ' . db_prefix() . 'clients.saleid=' . db_prefix() . 'sales.id) as is_converted',
            'zip',
        ]);

        $result = data_tables_init($aColumns, $sIndexColumn, $sTable, $join, $where, $additionalColumns);

        $output  = $result['output'];
        $rResult = $result['rResult'];
        // session_start();
        $ids = [];

        foreach ($rResult as $aRow) {
            $row = [];
            $ids[] = $aRow['id'];
            $row[] = '<div class="checkbox"><input type="checkbox" value="' . $aRow['id'] . '"><label></label></div>';

            $hrefAttr = 'href="' . admin_url('sales/index/' . $aRow['id']) . '" onclick="init_sale(' . $aRow['id'] . ');return false;"';
            $row[]    = '<a ' . $hrefAttr . '>' . $aRow['id'] . '</a>';

            $nameRow = '<a ' . $hrefAttr . '>' . e($aRow['name']) . '</a>';

            $nameRow .= '<div class="row-options">';
            $nameRow .= '<a ' . $hrefAttr . '>' . _l('view') . '</a>';

            $locked = false;

            if ($aRow['is_converted'] > 0) {
                $locked = ((!is_admin() && $lockAfterConvert == 1) ? true : false);
            }

            if (!$locked) {
                if(!staff_can('view_own',  'sales') ){
                    $nameRow .= ' | <a href="' . admin_url('sales/index/' . $aRow['id'] . '?edit=true') . '" onclick="init_sale(' . $aRow['id'] . ', true);return false;">' . _l('edit') . '</a>';
                }
            }

            if ($aRow['addedfrom'] == get_staff_user_id() || $has_permission_delete) {
                $nameRow .= ' | <a href="' . admin_url('sales/delete/' . $aRow['id']) . '" class="_delete text-danger">' . _l('delete') . '</a>';
            }
            $nameRow .= '</div>';


            $row[] = $nameRow;

            if (is_gdpr() && $consentsales == '1') {
                $consentHTML = '<p class="bold"><a href="#" onclick="view_sale_consent(' . $aRow['id'] . '); return false;">' . _l('view_consent') . '</a></p>';
                $consents    = $this->ci->gdpr_model->get_consent_purposes($aRow['id'], 'sale');

                foreach ($consents as $consent) {
                    $consentHTML .= '<p style="margin-bottom:0px;">' . e($consent['name']) . (!empty($consent['consent_given']) ? '<i class="fa fa-check text-success pull-right"></i>' : '<i class="fa fa-remove text-danger pull-right"></i>') . '</p>';
                }
                $row[] = $consentHTML;
            }
            $row[] = e($aRow['company']);

            $row[] = ($aRow['email'] != '' ? '<a href="mailto:' . e($aRow['email']) . '">' . e($aRow['email']) . '</a>' : '');

            $row[] = ($aRow['phonenumber'] != '' ? '<a href="tel:' . e($aRow['phonenumber']) . '">' . e($aRow['phonenumber']) . '</a>' : '');

            $base_currency = get_base_currency();
            $row[]         = e(($aRow['sale_value'] != 0 ? app_format_money($aRow['sale_value'], $base_currency->id) : ''));

            $row[] .= render_tags($aRow['tags']);

            $assignedOutput = '';
            if ($aRow['assigned'] != 0) {
                $full_name = e($aRow['assigned_firstname'] . ' ' . $aRow['assigned_lastname']);

                $assignedOutput = '<a data-toggle="tooltip" data-title="' . $full_name . '" href="' . admin_url('profile/' . $aRow['assigned']) . '">' . staff_profile_image($aRow['assigned'], [
                    'staff-profile-image-small',
                ]) . '</a>';

                // For exporting
                $assignedOutput .= '<span class="hide">' . $full_name . '</span>';
            }

            $row[] = $assignedOutput;

            if ($aRow['status_name'] == null) {
                if ($aRow['lost'] == 1) {
                    $outputStatus = '<span class="label label-danger">' . _l('sale_lost') . '</span>';
                } elseif ($aRow['junk'] == 1) {
                    $outputStatus = '<span class="label label-warning">' . _l('sale_junk') . '</span>';
                }
            } else {
                if ($aRow['status'] == 1) {
                    $outputStatus = '<span class="label label-danger">' . _l('sale_lost') . '</span>';
                }else {
                    $outputStatus = '<span class="sale-status-' . $aRow['status'] . ' label' . (empty($aRow['color']) ? ' label-default' : '') . '" style="color:' . $aRow['color'] . ';border:1px solid ' . adjust_hex_brightness($aRow['color'], 0.4) . ';background: ' . adjust_hex_brightness($aRow['color'], 0.04) . ';">' . e($aRow['status_name']);

                if (!$locked) {
                    $outputStatus .= '<div class="dropdown inline-block mleft5 table-export-exclude">';
                    $outputStatus .= '<a href="#" style="font-size:14px;vertical-align:middle;" class="dropdown-toggle text-dark" id="tablesalesStatus-' . $aRow['id'] . '" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">';
                    $outputStatus .= '<span data-toggle="tooltip" title="' . _l('ticket_single_change_status') . '"><i class="fa-solid fa-chevron-down tw-opacity-70"></i></span>';
                    $outputStatus .= '</a>';

                    $outputStatus .= '<ul class="dropdown-menu dropdown-menu-right" aria-labelledby="tablesalesStatus-' . $aRow['id'] . '">';

                    foreach ($statuses as $saleChangeStatus) {
                        if ($aRow['status'] != $saleChangeStatus['id']) {
                            $outputStatus .= '<li>
                          <a href="#" onclick="sale_mark_as(' . $saleChangeStatus['id'] . ',' . $aRow['id'] . '); return false;">
                             ' . e($saleChangeStatus['name']) . '
                          </a>
                       </li>';
                        }
                    }
                    
                    $outputStatus .= '</ul>';
                    $outputStatus .= '</div>';
                }
                $outputStatus .= '</span>';

                }
                

                
                
            }

            $row[] = $outputStatus;

            $row[] = e($aRow['source_name']);

            $row[] = ($aRow['lastcontact'] == '0000-00-00 00:00:00' || !is_date($aRow['lastcontact']) ? '' : '<span data-toggle="tooltip" data-title="' . e(_dt($aRow['lastcontact'])) . '" class="text-has-action is-date">' . e(time_ago($aRow['lastcontact'])) . '</span>');

            $row[] = '<span data-toggle="tooltip" data-title="' . e(_dt($aRow['dateadded'])) . '" class="text-has-action is-date">' . e(time_ago($aRow['dateadded'])) . '</span>';

            // Custom fields add values
            foreach ($customFieldsColumns as $customFieldColumn) {
                $row[] = (strpos($customFieldColumn, 'date_picker_') !== false ? _d($aRow[$customFieldColumn]) : $aRow[$customFieldColumn]);
            }

            $row['DT_RowId'] = 'sale_' . $aRow['id'];

            if ($aRow['assigned'] == get_staff_user_id()) {
                $row['DT_RowClass'] = 'info';
            }

            if (isset($row['DT_RowClass'])) {
                $row['DT_RowClass'] .= ' has-row-options';
            } else {
                $row['DT_RowClass'] = 'has-row-options';
            }

            $row = hooks()->apply_filters('sales_table_row_data', $row, $aRow);

            $output['aaData'][] = $row;
        }
        
        $_SESSION['saleids'] = $ids;
       
        // $_SESSION['sale_next_previous_data'] = array(
        //     'ids'=> $ids
        //   );
        return $output;
    })->setRules($rules);
