<?php defined('BASEPATH') or exit('No direct script access allowed');
/*
  Widget Name: Staff Breaks with Details
  Description: Display staff breaks with their full names
*/
?>
<script src="https://code.jquery.com/jquery-3.6.0.min.js"></script>
<?php
$fn_get_data = function () {
    // SQL query to get all staff who are currently on break and whose break hasn't ended
    $sql = "
        SELECT 
            Breaks.id,
            Breaks.staff_id,
            CONCAT(staff.firstname, ' ', staff.lastname) AS staff_fullname,
            Breaks.break_type,
            Breaks.break_minutes,
            Breaks.start_time,
            Breaks.end_time,
            Breaks.on_break,
            Breaks.mins_late,
            Breaks.created_at
        FROM 
            " . db_prefix() . "breaks Breaks
        JOIN 
            " . db_prefix() . "staff staff ON Breaks.staff_id = staff.staffid
        WHERE 
            Breaks.on_break = 1
        ORDER BY Breaks.created_at DESC
    ";

    return $this->db->query($sql)->result_array();
};

// Fetch initial data
$widget_data = $fn_get_data();

?>

<div class="widget widget-staff-breaks widget-<?= $widget['id'] ?>" data-widget-id="<?= $widget['id'] ?>">
    <div class="">
        <div class="panel_s">
            <div class="panel-body">
                <div class="widget-dragger"></div>

                <h4 class="pull-left mtop5"><?php echo _l('staff_breaks_with_details'); ?></h4>
                <div class="clearfix"></div>
                <div class="row mtop5">
                    <hr class="hr-panel-heading-dashboard">
                </div>

                <table class="table dataTable no-footer dtr-inline" id="staff-breaks-table">
                    <thead>
                        <th><?= _l('staff') ?></th>
                        <th><?= _l('break_type') ?></th>
                        <th><?= _l('break_minutes') ?></th>
                        <th><?= _l('start_time') ?></th>
                        <th><?= _l('end_time') ?></th>
                        <th><?= _l('on_break') ?></th>
                        <th><?= _l('mins_late') ?></th>
                        <th><?= _l('created_at') ?></th>
                    </thead>
                    <tbody id="staff-breaks-body">
                        <?php if (count($widget_data) > 0) { ?>
                            <?php foreach ($widget_data as $widget_row) { ?>
                                <tr data-staff-id="<?= $widget_row['staff_id'] ?>" data-end-time="<?= $widget_row['end_time'] ?>">
                                    <td><a href="<?= admin_url('staff/member/' . $widget_row['staff_id']) ?>"><?= $widget_row['staff_fullname'] ?></a></td>
                                    <td><?= $widget_row['break_type'] ?></td>
                                    <td><?= $widget_row['break_minutes'] ?></td>
                                    <td><?= $widget_row['start_time'] ?></td>
                                    <td><?= $widget_row['end_time'] ?></td>
                                    <td><?= $widget_row['on_break'] ? _l('yes') : _l('no') ?></td>
                                    <td><?= $widget_row['mins_late'] ?></td>
                                    <td><?= _dt($widget_row['created_at']) ?></td>
                                </tr>
                            <?php } ?>
                        <?php } else { ?>
                            <tr>
                                <td colspan="8"><?= _l('not_found') ?></td>
                            </tr>
                        <?php } ?>
                    </tbody>
                </table>
            </div>
        </div>
    </div>
</div>

<!-- CSS for blinking rows -->
<style>
    /* Blinking red effect */
    .blink-row {
        animation: blink-animation 1s infinite;
    }

    @keyframes blink-animation {
        0% { background-color: red; }
        50% { background-color: transparent; }
        100% { background-color: red; }
    }
</style>

<!-- JavaScript for polling and highlighting exceeded breaks -->
<script>
    // Poll every 5 seconds to refresh the widget and check for exceeded breaks
    setInterval(function() {
        // Fetch the latest widget data via AJAX
        $.ajax({
            url: "<?= current_url() ?>", // AJAX points to the current widget URL
            method: "GET",
            dataType: "json",
            success: function(response) {
                // Clear current table body
                $('#staff-breaks-body').empty();

                // Iterate over the new data and rebuild the table rows
                response.forEach(function(row) {
                    var onBreak = row.on_break ? "<?= _l('yes') ?>" : "<?= _l('no') ?>";
                    var newRow = `
                        <tr data-staff-id="${row.staff_id}" data-end-time="${row.end_time}">
                            <td><a href="<?= admin_url('staff/member/') ?>${row.staff_id}">${row.staff_fullname}</a></td>
                            <td>${row.break_type}</td>
                            <td>${row.break_minutes}</td>
                            <td>${row.start_time}</td>
                            <td>${row.end_time}</td>
                            <td>${onBreak}</td>
                            <td>${row.mins_late}</td>
                            <td>${row.created_at}</td>
                        </tr>
                    `;
                    $('#staff-breaks-body').append(newRow);
                });

                // After appending the rows, check for exceeded break times
                highlightExceededBreaks();
            },
            error: function(xhr, status, error) {
                console.log("Error fetching data: ", error);
            }
        });
    }, 5000); // Refresh widget every 5 seconds

    // Function to highlight rows where break time has been exceeded
    function highlightExceededBreaks() {
        const now = new Date();

        // Iterate over each row in the table
        $('#staff-breaks-table tbody tr').each(function() {
            var row = $(this);
            var endTime = row.data('end-time'); // Get the end time for each staff

            // Convert endTime to a Date object
            var endTimeDate = new Date(endTime);

            // Check if the current time exceeds the end time
            if (now > endTimeDate) {
                row.addClass('blink-row'); // Add blinking class if break is exceeded
            } else {
                row.removeClass('blink-row'); // Remove blinking if the break is not exceeded
            }
        });
    }

    // Run the highlight function initially to ensure any already exceeded breaks are highlighted
    highlightExceededBreaks();
</script>
