<?php

defined('BASEPATH') or exit('No direct script access allowed');
/*
  Widget Name: Sales and converted sales by staff
  Description: Sales and converted sales by staff
*/
?>

<?php

$fn_get_data = function () {
    // Validate 'period_from' input
    $widget_req_from = $this->input->get('period_from');
    if (!empty($widget_req_from)) {
        $widget_req_from = DateTime::createFromFormat('Y-m-d', $widget_req_from);
        if ($widget_req_from !== false) {
            $widget_req_from = $widget_req_from->format('Y-m-d');
        } else {
            $widget_req_from = null;
        }
    } else {
        $widget_req_from = null;
    }

    // Validate 'period_to' input
    $widget_req_to = $this->input->get('period_to');
    if (!empty($widget_req_to)) {
        $widget_req_to = DateTime::createFromFormat('Y-m-d', $widget_req_to);
        if ($widget_req_to !== false) {
            $widget_req_to = $widget_req_to->format('Y-m-d');
        } else {
            $widget_req_to = null;
        }
    } else {
        $widget_req_to = null;
    }

    $data_labels = [];
    $data_sales = [];
    $data_converted_sales = [];

    // SQL query to get sales and converted sales by staff
    $sql = "
        SELECT 
            TBLTotalSales.staff_id AS staff_id,
            TBLTotalSales.staff_fullname AS staff_fullname,
            TBLTotalSales.total_sales AS total_sales,
            TBLTotalConvertedSales.total_converted_sales AS total_converted_sales,
            (TBLTotalSales.total_sales + TBLTotalConvertedSales.total_converted_sales) AS total_rows
        FROM
            (SELECT 
                TBLStaff.staffid AS staff_id,
                CONCAT(TBLStaff.firstname, ' ', TBLStaff.lastname) AS staff_fullname,
                SUM(CASE
                    WHEN (TBLSales.id IS NOT NULL ";
    
    // Apply date filters for sales added
    if (isset($widget_req_from) && isset($widget_req_to)) {
        $sql .= "
                    AND TBLSales.dateadded >= '" . $widget_req_from . " 00:00:00'
                    AND TBLSales.dateadded <= '" . $widget_req_to . " 23:59:59'
        ";
    }
    $sql .= " 
                ) THEN 1 ELSE 0 END) AS total_sales
            FROM
            " . db_prefix() . "staff TBLStaff
            LEFT JOIN " . db_prefix() . "sales TBLSales ON TBLStaff.staffid = TBLSales.assigned
            GROUP BY staff_id, staff_fullname) TBLTotalSales
        INNER JOIN
            (SELECT 
                TBLStaff.staffid AS staff_id,
                SUM(CASE
                    WHEN (TBLSales.status = 1 ";
    
    // Apply date filters for converted sales
    if (isset($widget_req_from) && isset($widget_req_to)) {
        $sql .= "
                    AND TBLSales.date_converted >= '" . $widget_req_from . " 00:00:00'
                    AND TBLSales.date_converted <= '" . $widget_req_to . " 23:59:59'
        ";
    }
    $sql .= " 
                ) THEN 1 ELSE 0 END) AS total_converted_sales
            FROM
            " . db_prefix() . "staff TBLStaff
            LEFT JOIN " . db_prefix() . "sales TBLSales ON TBLSales.assigned = TBLStaff.staffid
            GROUP BY staff_id) TBLTotalConvertedSales ON TBLTotalSales.staff_id = TBLTotalConvertedSales.staff_id
        WHERE (TBLTotalSales.total_sales + TBLTotalConvertedSales.total_converted_sales) > 0
        ORDER BY total_rows DESC
        LIMIT 0 , 10
    ";

    // Fetch data from the database
    $data_rows = $this->db->query($sql)->result_array();

    foreach ($data_rows as $data_row) {
        array_push($data_labels, $data_row['staff_fullname']);
        array_push($data_sales, intval($data_row['total_sales']));
        array_push($data_converted_sales, intval($data_row['total_converted_sales']));
    }

    // Prepare chart data
    $chart = [
        'labels'   => $data_labels,
        'datasets' => [
            [
                'label' => _l('sales'),
                'backgroundColor' => 'rgba(37,155,35,0.2)',
                'borderColor' => '#84c529',
                'borderWidth' => 1,
                'tension' => false,
                'data' => $data_sales,
            ], [
                'label' => _l('converted_sales'),
                'backgroundColor' => 'rgba(197, 61, 169, 0.5)',
                'borderColor' => '#c53da9',
                'borderWidth' => 1,
                'tension' => false,
                'data' => $data_converted_sales,
            ],
        ],
    ];

    return $chart;
};

$widget_data = $fn_get_data();
?>

<div class="widget widget-leads-and-converted-leads-by-staff" data-widget-id="<?= $widget['id'] ?>">
    <div class="panel_s">
        <div class="panel-body">
            <div class="widget-dragger"></div>

            <h4 class="pull-left mtop5"><?php echo _l('sales_and_converted_sales_by_staff'); ?></h4>
            <div class="clearfix"></div>
            <div class="row mtop5">
                <hr class="hr-panel-heading-dashboard">
            </div>

            <div class="row">
                <div class="col-md-12">
                    <div class="relative max-height-400">
                        <canvas id="perfex_dashboard_widget_<?= $widget['id'] ?>_chart_data" class="animated fadeIn" height="400"></canvas>
                    </div>
                </div>
            </div>
        </div>
    </div>
</div>

<script>
  window['perfex_dashboard_widget_<?= $widget['id'] ?>_chart_data'] = <?= json_encode($widget_data); ?>;
</script>
