<?php
defined('BASEPATH') or exit('No direct script access allowed');
/*
  Widget Name: Top leads Converters
  Description: Display the top leads converters with their full names and conversion counts, filtered by a date range
*/
?>

<?php
$fn_get_data = function () {
    // Get the date range from the input
    $widget_req_from = $this->input->get('period_from');
    $widget_req_to = $this->input->get('period_to');

    if ($widget_req_from) {
        $widget_req_from = DateTime::createFromFormat('Y-m-d', $widget_req_from);
        $widget_req_from = $widget_req_from ? $widget_req_from->format('Y-m-d') : null;
    }
    if ($widget_req_to) {
        $widget_req_to = DateTime::createFromFormat('Y-m-d', $widget_req_to);
        $widget_req_to = $widget_req_to ? $widget_req_to->format('Y-m-d') : null;
    }

    // SQL query to get top leads converters where status = 1 (converted lead) and filter by date range if provided
    $sql = "
    SELECT 
        staff.staffid,
        CONCAT(staff.firstname, ' ', staff.lastname) AS staff_fullname,
        COUNT(leads.id) AS total_conversions,
        IFNULL(targets.target, 0) AS target,
        IFNULL((COUNT(leads.id) / targets.target) * 100, 0) AS achievement_percentage
    FROM 
        " . $this->db->dbprefix . "leads leads
    JOIN 
        " . $this->db->dbprefix . "staff staff ON leads.assigned = staff.staffid
    LEFT JOIN 
        " . $this->db->dbprefix . "targets targets ON staff.staffid = targets.user_id
    WHERE 
        leads.status = 1
    ";

    // Add date range filters
    if ($widget_req_from && $widget_req_to) {
        $sql .= " AND leads.last_status_change BETWEEN '" . $widget_req_from . "' AND '" . $widget_req_to . "'";
    } elseif ($widget_req_from) {
        $sql .= " AND leads.last_status_change >= '" . $widget_req_from . "'";
    } elseif ($widget_req_to) {
        $sql .= " AND leads.last_status_change <= '" . $widget_req_to . "'";
    }

    // Filter by target month if provided
    $target_month = $this->input->get('target_month');
    if (!empty($target_month)) {
        $sql .= " AND MONTH(targets.created_at) = '" . $target_month . "'";
    }

    $sql .= "
    GROUP BY 
        staff.staffid, targets.target
    ORDER BY 
        achievement_percentage DESC
    LIMIT 10
    ";

    return $this->db->query($sql)->result_array();
};

// Get top leads converters data
$widget_data = $fn_get_data();
?>

<div class="widget widget-top-leads-converters widget-<?= $widget['id'] ?>" data-widget-id="<?= $widget['id'] ?>">
    <div class="">
        <div class="">
            <h4 class="pull-left mtop5"><?php echo _l('Top leads Rankers'); ?></h4>
            <div class="clearfix"></div>
            <div class="row mtop5">
                <hr class="hr-panel-heading-dashboard">
            </div>

            <div class="leaderboard">
                <div class="leaderboard-header">
                    <div class="header-rank">#</div>
                    <div class="header-name">Employee</div>
                    <div class="header-conversions">Conversions</div>
                    <div class="header-target">Target</div>
                    <div class="header-achievement">Win Rate</div>
                </div>
                <?php if (count($widget_data) > 0) { ?>
                    <?php foreach ($widget_data as $index => $widget_row) { 
                        $achievement_percentage = min(100, $widget_row['achievement_percentage']);
                        $bar_color = '#FF5F57'; // Default red
                        if ($achievement_percentage >= 70) {
                            $bar_color = '#4CAF50'; // Green for high achievers
                        } elseif ($achievement_percentage >= 50) {
                            $bar_color = '#FFC107'; // Amber for medium achievers
                        } elseif ($achievement_percentage >= 30) {
                            $bar_color = '#FF9800'; // Orange for low achievers
                        }
                    ?>
                        <div class="leaderboard-item">
                            <div class="item-rank"><?= $index + 1 ?></div>
                            <div class="item-employee">
                                <img src="<?= base_url('uploads/staff_images/' . $widget_row['staffid'] . '.jpg') ?>" alt="<?= $widget_row['staff_fullname'] ?>" onerror="this.src='https://via.placeholder.com/40'">
                                <span><?= $widget_row['staff_fullname'] ?></span>
                            </div>
                            <div class="item-conversions"><?= $widget_row['total_conversions'] ?></div>
                            <div class="item-target"><?= $widget_row['target'] ?></div>
                            <div class="item-achievement">
                                <span class="achievement-percentage"><?= round($widget_row['achievement_percentage'], 2) ?>%</span>
                                <div class="progress-bar">
                                    <div class="progress" style="width: <?= $achievement_percentage ?>%; background-color: <?= $bar_color ?>;"></div>
                                </div>
                            </div>
                        </div>
                    <?php } ?>
                <?php } else { ?>
                    <div class="no-data"><?= _l('not_found') ?></div>
                <?php } ?>
            </div>
        </div>
    </div>
</div>

<style>
/* Base styling */
body {
    background-color: #191919;
    font-family: Arial, sans-serif;
    color: #E2E8F0;
}

.leaderboard {
    width: 100%;
    background-color: #191919;
    border-radius: 10px;
    padding: 20px;
}

/* Header and item structure using grid layout */
.leaderboard-header,
.leaderboard-item {
    display: grid;
    grid-template-columns: 5% 35% 15% 15% 30%;
    align-items: center;
    padding: 10px 0;
}

.leaderboard-header {
    border-bottom: 1px solid #4A5568;
    font-weight: bold;
    color: #A0AEC0;
}


.item-rank, .item-employee, .item-conversions, .item-target, .item-achievement {
    padding: 0 10px;
}

/*.header-name, .header-conversions, .header-target, .header-achievement{*/
    
/* align-self: center;*/
/* background-color:green*/
/*}*/


.header-conversions {
    text-align: center;
}

.header-target {
    text-align: center;
}

.header-achievement {
    text-align: right;
}


.item-rank, .item-conversions, .item-target {
    text-align: center;
}

.item-employee {
    display: flex;
    align-items: center;
    gap: 10px;
}

.item-employee img {
    width: 32px;
    height: 32px;
    border-radius: 50%;
    object-fit: cover;
}

.item-achievement {
    display: flex;
    align-items: center;
    justify-content: flex-end;
    gap: 10px;
}

.achievement-percentage {
    font-weight: bold;
}

.progress-bar {
    width: 60%;
    height: 12px;
    background-color: #4A5568;
    border-radius: 6px;
    overflow: hidden;
    display: inline-block;
}

.progress {
    height: 100%;
    border-radius: 6px;
}
</style>

<script>
    // Optional: Add custom JavaScript if needed
</script>
