<?php
defined('BASEPATH') or exit('No direct script access allowed');
/*
  Widget Name: Top retentions Converters
  Description: Display the top retentions converters with their full names and conversion counts, filtered by a date range
*/
?>

<?php
$fn_get_data = function () {
    // Get the date range from the input
    $widget_req_from = $this->input->get('period_from');
    if (!empty($widget_req_from)) {
        $widget_req_from = DateTime::createFromFormat('Y-m-d', $widget_req_from);
        if ($widget_req_from !== false) {
            $widget_req_from = $widget_req_from->format('Y-m-d');
        } else {
            $widget_req_from = null;
        }
    } else {
        $widget_req_from = null;
    }

    $widget_req_to = $this->input->get('period_to');
    if (!empty($widget_req_to)) {
        $widget_req_to = DateTime::createFromFormat('Y-m-d', $widget_req_to);
        if ($widget_req_to !== false) {
            $widget_req_to = $widget_req_to->format('Y-m-d');
        } else {
            $widget_req_to = null;
        }
    } else {
        $widget_req_to = null;
    }

    // SQL query to get top retentions converters where status = 1 (converted retention) and filter by date range if provided
    $sql = "
    SELECT 
        staff.staffid,
        CONCAT(staff.firstname, ' ', staff.lastname) AS staff_fullname,
        COUNT(retentions.id) AS total_conversions,
        IFNULL(targets.target, 0) AS target,
        IFNULL((COUNT(retentions.id) / targets.target) * 100, 0) AS achievement_percentage
    FROM 
        " . $this->db->dbprefix . "retentions retentions
    JOIN 
        " . $this->db->dbprefix . "staff staff ON retentions.assigned = staff.staffid
    LEFT JOIN 
        " . $this->db->dbprefix . "targets targets ON staff.staffid = targets.user_id
    WHERE 
        retentions.status = 1
";

// Add date range filters
if ($widget_req_from && $widget_req_to) {
    $sql .= " AND retentions.last_status_change BETWEEN '" . $widget_req_from . "' AND '" . $widget_req_to . "'";
} elseif ($widget_req_from) {
    $sql .= " AND retentions.last_status_change >= '" . $widget_req_from . "'";
} elseif ($widget_req_to) {
    $sql .= " AND retentions.last_status_change <= '" . $widget_req_to . "'";
}

// Filter by target month if provided
$target_month = $this->input->get('target_month');
if (!empty($target_month)) {
    $sql .= " AND MONTH(targets.created_at) = '" . $target_month . "'";
}

$sql .= "
    GROUP BY 
        staff.staffid
    ORDER BY 
        achievement_percentage DESC
    LIMIT 10
";

return $this->db->query($sql)->result_array();
};

// Get top retentions converters data
$widget_data = $fn_get_data();

?>

<style>
/* Highlighting top rank */
.top-rank-row {
    font-weight: bold;
    color: green;
}

/* Leaderboard styles */

.leaderboard {
    width: 100%;
    background-color: #FFD54F;
    border-radius: 10px;
    padding: 20px;
    box-shadow: 0 4px 8px rgba(0, 0, 0, 0.2);
}

.leaderboard-item {
    display: flex;
    align-items: center;
    margin-bottom: 15px;
}

.leaderboard-item img {
    width: 40px;
    height: 40px;
    border-radius: 50%;
    margin-right: 15px;
}

.leaderboard-content {
    flex: 1;
    display: flex;
    flex-direction: column;
    justify-content: center;
}

.leaderboard-content .name {
    font-weight: bold;
    font-size: 14px;
}

.leaderboard-content .points {
    color: #555;
    font-size: 12px;
    margin-bottom: 5px;
}

.progress-bar {
    width: 100%;
    height: 8px;
    background-color: #FFF3E0;
    border-radius: 4px;
    overflow: hidden;
    margin-top: 5px;
}

.progress {
    height: 100%;
    background-color: #FFA726;
}




/* Highlighting top rank */
.top-rank-row {
    font-weight: bold;
    color: green;
}

/* Leaderboard styles */
.leaderboard {
    background-color: #FFD54F;
    border-radius: 10px;
    padding: 20px;
    box-shadow: 0 4px 8px rgba(0, 0, 0, 0.2);
}

.leaderboard-item {
    display: flex;
    align-items: center;
    margin-bottom: 15px;
}

.leaderboard-item img {
    width: 40px;
    height: 40px;
    border-radius: 50%;
    margin-right: 15px;
}

.leaderboard-content {
    flex: 1;
    display: flex;
    flex-direction: column;
    justify-content: center;
}

.leaderboard-content .name {
    font-weight: bold;
    font-size: 14px;
}

.leaderboard-content .points,
.leaderboard-content .target,
.leaderboard-content .achievement {
    color: #555;
    font-size: 12px;
    margin-top: 2px;
}

.progress-bar {
    width: 100%;
    height: 8px;
    background-color: #FFF3E0;
    border-radius: 4px;
    overflow: hidden;
    margin-top: 5px;
}

.progress {
    height: 100%;
    background-color: #FFA726;
}
</style>
</style>

<div class="widget widget-top-retentions-converters widget-<?= $widget['id'] ?>" data-widget-id="<?= $widget['id'] ?>">
    <div class="">
        <div class="">
            <div class="">
                <div class="widget-dragger"></div>

                <h4 class="pull-left mtop5"><?php echo _l('Top retentions Rankers'); ?></h4>
                <div class="clearfix"></div>
                <div class="row mtop5">
                    <hr class="hr-panel-heading-dashboard">
                </div>

                <div class="leaderboard">
                    <?php if (count($widget_data) > 0) { ?>
                        <?php foreach ($widget_data as $index => $widget_row) { ?>
                            <div class="leaderboard-item <?= $index == 0 ? 'top-rank-row' : '' ?>">
                                <!-- Staff image (placeholder or actual) -->
                                <img src="<?= base_url('uploads/staff_images/' . $widget_row['staffid'] . '.jpg') ?>" alt="<?= $widget_row['staff_fullname'] ?>" onerror="this.src='https://via.placeholder.com/40'">
                                <div class="leaderboard-content">
                                    <div class="name"><?= ($index + 1) . '. ' . $widget_row['staff_fullname'] ?></div>
                                   
                                    <div class="progress-bar-wrapper">
                                        <div class="progress-bar">
                                            <div class="progress" style="width: <?= min(100, $widget_row['achievement_percentage']) ?>%;"></div>
                                        </div>
                                        <div class="conversion-info">
                                            <span class="conversion-amount"><?= $widget_row['total_conversions'] ?> Conversions</span>
                                            <span class="achievement-percentage"><?= round($widget_row['achievement_percentage'], 2) ?>% Conversion Rate</span>
                                            <div class="target">Target: <?= $widget_row['target'] ?></div>
                                        </div>
                                    </div>
                                   
                                </div>
                            </div>
                        <?php } ?>
                    <?php } else { ?>
                        <div class="no-data"><?= _l('not_found') ?></div>
                    <?php } ?>
                </div>
            </div>
        </div>
    </div>
</div>

<style>
/* Highlighting top rank */
.top-rank-row {
    font-weight: bold;
    color: green;
}

/* Leaderboard styles */
.leaderboard {
    background-color: #FFD54F;
    border-radius: 10px;
    padding: 20px;
    box-shadow: 0 4px 8px rgba(0, 0, 0, 0.2);
}

.leaderboard-item {
    display: flex;
    align-items: center;
    margin-bottom: 15px;
}

.leaderboard-item img {
    width: 40px;
    height: 40px;
    border-radius: 50%;
    margin-right: 15px;
}

.leaderboard-content {
    flex: 1;
    display: flex;
    flex-direction: column;
    justify-content: center;
}

.leaderboard-content .name {
    font-weight: bold;
    font-size: 14px;
}

.leaderboard-content .points,
.leaderboard-content .target {
    color: #555;
    font-size: 12px;
    margin-top: 2px;
}

.progress-bar-wrapper {
    display: flex;
    align-items: center;
    margin-top: 5px;
}

.progress-bar {
    width: 80%;
    height: 8px;
    background-color: #FFF3E0;
    border-radius: 4px;
    overflow: hidden;
    margin-right: 10px;
}

.progress {
    height: 100%;
    background-color: #FFA726;
}

.conversion-info {
    display: flex;
    flex-direction: column;
    align-items: flex-start;
    font-size: 12px;
    color: #555;
}

.conversion-info .conversion-amount,
.conversion-info .achievement-percentage {
    margin: 0;
}
</style>



<script>
    // Optional: Add custom JavaScript if needed
</script>
