
$(document).ready(function () {

    const NOTIFICATION_INTERVAL = 30000; // 30 seconds
    const NOTIFICATION_DURATION = 40000; // 40 seconds
    const ADMIN_URL = admin_url + "utilities/get_event_data/";
    const NOTIFICATION_FILE = '<?php echo base_url('uploads/ringtone.mp3'); ?>';

    let intervalId;
    let notification;

    function fetchEventData() {
        $.get(ADMIN_URL)
            .done(handleResponse)
            .fail((jqXHR, textStatus, errorThrown) => {
                console.error('Request failed:', textStatus, errorThrown);
            });
    }

    function handleResponse(response) {
        try {
            const data = typeof response === 'string' ? JSON.parse(response) : response;
            if (data.status === 'success' && data.data) {
                playNotification();
                updateModalContent(data.data);
                showModal();
                scheduleModalClose();
            }
        } catch (e) {
            console.error('Failed to parse JSON response:', e);
        }
    }

    function playNotification() {
        if (!notification) {
            notification = new Audio(NOTIFICATION_FILE);
            notification.loop = false;
        }
        notification.play().catch(error => {
            console.error('Error playing audio:', error);
        });
    }

    function updateModalContent(data) {
        $('#eventTitle').text(data.title || 'No title');
        $('#eventStart').text(data.start || 'No start time');
    }

    function showModal() {
        $('#newOrderAdminModal').modal({
            backdrop: 'static',
            keyboard: true
        });
    }

    function scheduleModalClose() {
        setTimeout(() => {
            stopNotification();
            $('#newOrderAdminModal').modal('hide');
        }, NOTIFICATION_DURATION);
    }

    function startNotificationInterval() {
        intervalId = setInterval(fetchEventData, NOTIFICATION_INTERVAL);
    }

    function stopNotification() {
        if (notification) {
            notification.pause();
            notification.currentTime = 0;
        }
        clearInterval(intervalId);
    }

    startNotificationInterval();

    document.getElementById('closeModalButton').addEventListener('click', function() {
        stopNotification();
        $('#newOrderAdminModal').modal('hide');
    });

});